import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestFileDescriptor(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFFileDescriptorRef)

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFFileDescriptorGetTypeID(), int)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFFileDescriptorReadCallBack, 1 << 0)
        self.assertEqual(CoreFoundation.kCFFileDescriptorWriteCallBack, 1 << 1)

    def testInspection(self):
        def callout(fd, types, context):
            pass

        class Context:
            pass

        context = Context()
        fd = CoreFoundation.CFFileDescriptorCreate(None, 0, False, callout, context)
        self.assertIsInstance(fd, CoreFoundation.CFFileDescriptorRef)
        self.assertEqual(CoreFoundation.CFFileDescriptorGetNativeDescriptor(fd), 0)
        ctx = CoreFoundation.CFFileDescriptorGetContext(fd, None)
        self.assertIs(ctx, context)
        CoreFoundation.CFFileDescriptorEnableCallBacks(
            fd, CoreFoundation.kCFFileDescriptorReadCallBack
        )
        CoreFoundation.CFFileDescriptorDisableCallBacks(
            fd,
            CoreFoundation.kCFFileDescriptorReadCallBack
            | CoreFoundation.kCFFileDescriptorWriteCallBack,
        )

        rls = CoreFoundation.CFFileDescriptorCreateRunLoopSource(None, fd, 0)
        self.assertIsInstance(rls, CoreFoundation.CFRunLoopSourceRef)
        self.assertTrue(CoreFoundation.CFFileDescriptorIsValid(fd))
        CoreFoundation.CFFileDescriptorInvalidate(fd)
        self.assertFalse(CoreFoundation.CFFileDescriptorIsValid(fd))

        self.assertResultIsBOOL(CoreFoundation.CFFileDescriptorIsValid)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTableViewRowAction(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTableViewRowActionStyleRegular, 0)
        self.assertEqual(AppKit.NSTableViewRowActionStyleDestructive, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestAE(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSAppleScriptErrorMessage, str)
        self.assertIsInstance(Foundation.NSAppleScriptErrorNumber, str)
        self.assertIsInstance(Foundation.NSAppleScriptErrorAppName, str)
        self.assertIsInstance(Foundation.NSAppleScriptErrorBriefMessage, str)
        self.assertIsInstance(Foundation.NSAppleScriptErrorRange, str)

    def testOutput(self):
        obj = Foundation.NSAppleScript.alloc().initWithSource_(
            'tell application Terminal to do Xscript "ls -l"'
        )
        ok, error = obj.compileAndReturnError_(None)
        self.assertIs(ok, False)
        self.assertIsInstance(error, Foundation.NSDictionary)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSAppleScript.isCompiled)
        self.assertResultIsBOOL(Foundation.NSAppleScript.compileAndReturnError_)
        self.assertArgIsOut(Foundation.NSAppleScript.compileAndReturnError_, 0)
        self.assertArgIsOut(Foundation.NSAppleScript.executeAndReturnError_, 0)
        self.assertArgIsOut(Foundation.NSAppleScript.executeAppleEvent_error_, 1)
        self.assertArgIsOut(Foundation.NSAppleScript.initWithContentsOfURL_error_, 1)



import time
import objc

import CoreFoundation
from Foundation import NSDate
from PyObjCTools.TestSupport import TestCase


class TestDate(TestCase):
    def testTypes(self):
        try:
            cls = objc.lookUpClass("__NSDate")
            self.assertIs(cls, CoreFoundation.CFDateRef)
        except objc.error:
            try:
                cls = objc.lookUpClass("__NSCFDate")
                self.assertIs(cls, CoreFoundation.CFDateRef)
            except objc.error:
                self.assertIsCFType(CoreFoundation.CFDateRef)

        try:
            cls = objc.lookUpClass("NSTimeZone")
            self.assertIs(cls, CoreFoundation.CFTimeZoneRef)
        except objc.error:
            self.assertIsCFType(CoreFoundation.CFTimeZoneRef)

    def testTypeID(self):
        v = CoreFoundation.CFDateGetTypeID()
        self.assertIsInstance(v, int)

    def testConstants(self):
        self.assertIsInstance(CoreFoundation.kCFAbsoluteTimeIntervalSince1970, float)
        self.assertIsInstance(CoreFoundation.kCFAbsoluteTimeIntervalSince1904, float)
        self.assertEqual(CoreFoundation.kCFGregorianUnitsYears, (1 << 0))
        self.assertEqual(CoreFoundation.kCFGregorianUnitsMonths, (1 << 1))
        self.assertEqual(CoreFoundation.kCFGregorianUnitsDays, (1 << 2))
        self.assertEqual(CoreFoundation.kCFGregorianUnitsHours, (1 << 3))
        self.assertEqual(CoreFoundation.kCFGregorianUnitsMinutes, (1 << 4))
        self.assertEqual(CoreFoundation.kCFGregorianUnitsSeconds, (1 << 5))
        self.assertEqual(CoreFoundation.kCFGregorianAllUnits, 0x00FFFFFF)

    def testStructs(self):
        v = CoreFoundation.CFGregorianDate()
        self.assertHasAttr(v, "year")
        self.assertHasAttr(v, "month")
        self.assertHasAttr(v, "day")
        self.assertHasAttr(v, "hour")
        self.assertHasAttr(v, "minute")
        self.assertHasAttr(v, "second")

        v = CoreFoundation.CFGregorianUnits()
        self.assertHasAttr(v, "years")
        self.assertHasAttr(v, "months")
        self.assertHasAttr(v, "days")
        self.assertHasAttr(v, "hours")
        self.assertHasAttr(v, "minutes")
        self.assertHasAttr(v, "seconds")

    def testAbsoluteTime(self):
        v = CoreFoundation.CFAbsoluteTimeGetCurrent()
        self.assertIsInstance(v, float)
        self.assertLessThan(
            abs(v - time.time() + CoreFoundation.kCFAbsoluteTimeIntervalSince1970), 1.0
        )

    def testCreation(self):
        now = CoreFoundation.CFAbsoluteTimeGetCurrent()
        dt = CoreFoundation.CFDateCreate(None, now)
        self.assertIsInstance(dt, NSDate)

    def testInspection(self):
        now = CoreFoundation.CFAbsoluteTimeGetCurrent()
        nowtm = time.localtime()
        dt = CoreFoundation.CFDateCreate(None, now)
        self.assertIsInstance(dt, NSDate)
        self.assertEqual(CoreFoundation.CFDateGetAbsoluteTime(dt), now)
        dt2 = CoreFoundation.CFDateCreate(None, now + 1000)
        self.assertEqual(CoreFoundation.CFDateGetTimeIntervalSinceDate(dt2, dt), 1000)
        cmp = CoreFoundation.CFDateCompare(dt, dt2, 0)
        self.assertLessThan(cmp, 0)
        dt = CoreFoundation.CFGregorianDate()
        dt.month = 12
        self.assertIs(
            CoreFoundation.CFGregorianDateIsValid(
                dt, CoreFoundation.kCFGregorianUnitsMonths
            ),
            True,
        )
        dt.month = 99
        self.assertIs(
            CoreFoundation.CFGregorianDateIsValid(
                dt, CoreFoundation.kCFGregorianUnitsMonths
            ),
            False,
        )
        tz = CoreFoundation.CFTimeZoneCopyDefault()
        dt.year = 2008
        dt.day = 1
        dt.month = 1
        abstime = CoreFoundation.CFGregorianDateGetAbsoluteTime(dt, tz)
        self.assertIsInstance(abstime, float)
        dt = CoreFoundation.CFAbsoluteTimeGetGregorianDate(now, tz)
        self.assertEqual(dt.year, nowtm.tm_year)
        self.assertEqual(dt.month, nowtm.tm_mon)
        self.assertEqual(dt.day, nowtm.tm_mday)
        self.assertEqual(dt.hour, nowtm.tm_hour)
        self.assertEqual(dt.minute, nowtm.tm_min)
        units = CoreFoundation.CFGregorianUnits(days=1)
        stamp = CoreFoundation.CFAbsoluteTimeAddGregorianUnits(now, tz, units)
        self.assertEqual(stamp - now, 24 * 3600)
        units = CoreFoundation.CFAbsoluteTimeGetDifferenceAsGregorianUnits(
            stamp, now, tz, CoreFoundation.kCFGregorianAllUnits
        )
        self.assertEqual(units.days, 1)
        self.assertEqual(units.seconds, 0)
        v = CoreFoundation.CFAbsoluteTimeGetDayOfWeek(now, tz)
        self.assertEqual(v, nowtm.tm_wday + 1)
        v = CoreFoundation.CFAbsoluteTimeGetDayOfYear(now, tz)
        self.assertEqual(v, nowtm.tm_yday)
        v = CoreFoundation.CFAbsoluteTimeGetWeekOfYear(now, tz)
        self.assertIsInstance(v, int)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSearchToolbarItem(TestCase):
    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(
            AppKit.NSSearchToolbarItem.resignsFirstResponderWithCancel
        )
        self.assertArgIsBOOL(
            AppKit.NSSearchToolbarItem.setResignsFirstResponderWithCancel_, 0
        )



"""
No tests for CoreFoundation.CFPlugin: these API's are unsupported for the moment.
"""
import CoreFoundation
from PyObjCTools.TestSupport import TestCase

symbols = [
    # From CoreFoundation.CFPluginCOM.h:
    "IUnknownVTbl",
    "IS_ERROR",
    "HRESULT_CODE",
    "HRESULT_FACILITY",
    "SEVERITY_SUCCESS",
    "SEVERITY_ERROR",
    "MAKE_HRESULT",
    "S_OK",
    "S_FALSE",
    "E_UNEXPECTED",
    "E_NOTIMPL",
    "E_OUTOFMEMORY",
    "E_INVALIDARG",
    "E_NOINTERFACE",
    "E_POINTER",
    "E_HANDLE",
    "E_ABORT",
    "E_FAIL",
    "E_ACCESSDENIED",
    # From CoreFoundation.CFPlugin.h:
    "kCFPlugInDynamicRegistrationKey",
    "kCFPlugInDynamicRegisterFunctionKey",
    "kCFPlugInUnloadFunctionKey",
    "kCFPlugInFactoriesKey",
    "kCFPlugInTypesKey",
    "CFPlugInGetTypeID",
    "CFPlugInCreate",
    "CFPlugInGetBundle",
    "CFPlugInSetLoadOnDemand",
    "CFPlugInIsLoadOnDemand",
    "CFPlugInFindFactoriesForPlugInType",
    "CFPlugInFindFactoriesForPlugInTypeInPlugIn",
    "CFPlugInInstanceCreate",
    "CFPlugInRegisterFactoryFunction",
    "CFPlugInRegisterFactoryFunctionByName",
    "CFPlugInUnregisterFactory",
    "CFPlugInRegisterPlugInType",
    "CFPlugInUnregisterPlugInType",
    "CFPlugInAddInstanceForFactory",
    "CFPlugInRemoveInstanceForFactory",
    "CFPlugInInstanceGetInterfaceFunctionTable",
    "CFPlugInInstanceGetFactoryName",
    "CFPlugInInstanceGetInstanceData",
    "CFPlugInInstanceGetTypeID",
    "CFPlugInInstanceCreateWithInstanceDataSize",
]


class TestPluginNotSuppported(TestCase):
    def testUnsupported(self):
        for sym in symbols:
            with self.subTest(sym):
                if hasattr(CoreFoundation, sym):
                    self.fail(f"Unsupported symbol present: {sym}")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSCollectionViewLayout(TestCase):
    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(AppKit.NSCollectionElementCategoryItem, 0)
        self.assertEqual(AppKit.NSCollectionElementCategorySupplementaryView, 1)
        self.assertEqual(AppKit.NSCollectionElementCategoryDecorationView, 2)
        self.assertEqual(AppKit.NSCollectionElementCategoryInterItemGap, 3)

        self.assertEqual(AppKit.NSCollectionUpdateActionInsert, 0)
        self.assertEqual(AppKit.NSCollectionUpdateActionDelete, 1)
        self.assertEqual(AppKit.NSCollectionUpdateActionReload, 2)
        self.assertEqual(AppKit.NSCollectionUpdateActionMove, 3)
        self.assertEqual(AppKit.NSCollectionUpdateActionNone, 4)

        self.assertIsInstance(AppKit.NSCollectionElementKindInterItemGapIndicator, str)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSCollectionViewLayoutAttributes.isHidden)
        self.assertArgIsBOOL(AppKit.NSCollectionViewLayoutAttributes.setHidden_, 0)

        self.assertResultIsBOOL(
            AppKit.NSCollectionViewLayoutInvalidationContext.invalidateEverything
        )
        self.assertResultIsBOOL(
            AppKit.NSCollectionViewLayoutInvalidationContext.invalidateDataSourceCounts
        )

        self.assertResultIsBOOL(
            AppKit.NSCollectionViewLayout.shouldInvalidateLayoutForBoundsChange_
        )
        self.assertResultIsBOOL(
            AppKit.NSCollectionViewLayout.shouldInvalidateLayoutForPreferredLayoutAttributes_withOriginalAttributes_  # noqa: B950
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSharingServiceHelper(AppKit.NSObject):
    def showRelativeToRect_ofView_preferredEdge_(self, r, v, e):
        pass


class TestNSSharingService(TestCase):
    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSSharingServiceNameCloudSharing, str)

        self.assertEqual(AppKit.NSCloudKitSharingServiceStandard, 0)
        self.assertEqual(AppKit.NSCloudKitSharingServiceAllowPublic, 1 << 0)
        self.assertEqual(AppKit.NSCloudKitSharingServiceAllowPrivate, 1 << 1)
        self.assertEqual(AppKit.NSCloudKitSharingServiceAllowReadOnly, 1 << 4)
        self.assertEqual(AppKit.NSCloudKitSharingServiceAllowReadWrite, 1 << 5)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(AppKit.NSSharingServiceNamePostOnTencentWeibo, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostOnLinkedIn, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameUseAsFacebookProfileImage, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameUseAsLinkedInProfileImage, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSSharingServiceNamePostOnFacebook, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostOnTwitter, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostOnSinaWeibo, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameComposeEmail, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameComposeMessage, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameSendViaAirDrop, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameAddToSafariReadingList, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameAddToIPhoto, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameAddToAperture, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameUseAsTwitterProfileImage, str)
        self.assertIsInstance(AppKit.NSSharingServiceNameUseAsDesktopPicture, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostImageOnFlickr, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostVideoOnVimeo, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostVideoOnYouku, str)
        self.assertIsInstance(AppKit.NSSharingServiceNamePostVideoOnTudou, str)

        self.assertEqual(AppKit.NSSharingContentScopeItem, 0)
        self.assertEqual(AppKit.NSSharingContentScopePartial, 1)
        self.assertEqual(AppKit.NSSharingContentScopeFull, 2)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBlock(
            AppKit.NSSharingService.initWithTitle_image_alternateImage_handler_, 3, b"v"
        )
        self.assertResultIsBOOL(AppKit.NSSharingService.canPerformWithItems_)

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSSharingServiceDelegate")
        objc.protocolNamed("NSSharingServicePickerDelegate")

    def testProtocol(self):
        self.assertArgHasType(
            TestNSSharingServiceHelper.showRelativeToRect_ofView_preferredEdge_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSSharingServiceHelper.showRelativeToRect_ofView_preferredEdge_,
            2,
            objc._C_NSUInteger,
        )

    @min_sdk_level("10.12")
    def testProtocolObjects10_12(self):
        objc.protocolNamed("NSCloudSharingServiceDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSRulerMarker(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSRulerMarker.isMovable)
        self.assertArgIsBOOL(AppKit.NSRulerMarker.setMovable_, 0)
        self.assertResultIsBOOL(AppKit.NSRulerMarker.isRemovable)
        self.assertArgIsBOOL(AppKit.NSRulerMarker.setRemovable_, 0)
        self.assertResultIsBOOL(AppKit.NSRulerMarker.isDragging)
        self.assertResultIsBOOL(AppKit.NSRulerMarker.trackMouse_adding_)
        self.assertArgIsBOOL(AppKit.NSRulerMarker.trackMouse_adding_, 1)



#
#  FilteringControllerDocument.py
#  FilteringController
#
#  Converted by u.fiedler on 05.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Cocoa import NSDocument, NSKeyedArchiver, NSKeyedUnarchiver
from objc import super


class FilteringControllerDocument(NSDocument):
    peopleController = objc.IBOutlet()

    def init(self):
        self = super().init()
        if self is None:
            return None
        self._k_people = []
        return self

    def windowNibName(self):
        return "FilteringControllerDocument"

    def windowControllerDidLoadNib_(self, controller):
        super().windowControllerDidLoadNib_(controller)

    def dataRepresentationOfType_(self, aType):
        return NSKeyedArchiver.archivedDataWithRootObject_(self._k_people)

    def loadDataRepresentation_ofType_(self, data, aType):
        self.setPeople_(NSKeyedUnarchiver.unarchiveObjectWithData_(data))
        return True

    # indexed accessors

    def people(self):
        return self._k_people

    def setPeople_(self, people):
        self._k_people[:] = people

    @objc.accessor
    def countOfPeople(self):
        return len(self._k_people)

    @objc.accessor
    def objectInPeopleAtIndex_(self, idx):
        return self._k_people[idx]

    @objc.accessor
    def insertObject_inPeopleAtIndex_(self, obj, idx):
        self._k_people.insert(idx, obj)

    @objc.accessor
    def removeObjectFromPeopleAtIndex_(self, idx):
        del self._k_people[idx]

    @objc.accessor
    def replaceObjectInPeopleAtIndex_withObject_(self, idx, obj):
        self._k_people[idx] = obj



#
# Tests for the struct-wrappers for NSPoint, NSSize, NSRange and NSRect.
#
import operator

import Foundation
from PyObjCTools.TestSupport import TestCase


def do_set_slice(op, start, stop, value):
    op[start:stop] = value


def do_del_slice(op, start, stop):
    del op[start:stop]


class TestNSPoint(TestCase):
    def testConstructor(self):
        p = Foundation.NSPoint()
        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 0)
        self.assertEqual(p.y, 0)

        p = Foundation.NSPoint(1, 2)
        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 1)
        self.assertEqual(p.y, 2)
        self.assertEqual(p[0], 1)
        self.assertEqual(p[1], 2)

        p = Foundation.NSPoint(y=1, x=2)
        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 2)
        self.assertEqual(p.y, 1)
        self.assertEqual(p[1], 1)
        self.assertEqual(p[0], 2)

        self.assertRaises(TypeError, Foundation.NSPoint, 1, 2, 3)
        self.assertRaises(TypeError, Foundation.NSPoint, 1, 2, y=3)
        self.assertRaises(TypeError, Foundation.NSPoint, 1, x=3)
        self.assertRaises(TypeError, Foundation.NSPoint, x=3, z=4)

    def testMakePoint(self):
        p = Foundation.NSMakePoint(1, 2)
        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 1)
        self.assertEqual(p.y, 2)

    def testHash(self):
        p = Foundation.NSMakePoint(1, 2)
        self.assertRaises(TypeError, hash, p)

    def testCompare(self):
        p = Foundation.NSMakePoint(1, 2)
        q = Foundation.NSMakePoint(2, 3)
        P = (1, 2)
        Q = (2, 3)

        self.assertTrue(not (p < p))
        self.assertTrue(not (p < P))
        self.assertTrue(p < q)
        self.assertTrue(p < Q)

        self.assertTrue(p <= p)
        self.assertTrue(p <= P)
        self.assertTrue(p <= q)
        self.assertTrue(p <= Q)

        self.assertTrue(p == p)
        self.assertTrue(p == P)
        self.assertTrue(not (p == q))
        self.assertTrue(not (p == Q))

        self.assertTrue(p != q)
        self.assertTrue(p != Q)
        self.assertTrue(not (p != p))
        self.assertTrue(not (p != P))

        self.assertTrue(q >= p)
        self.assertTrue(q >= P)
        self.assertTrue(q >= q)
        self.assertTrue(q >= Q)

        self.assertTrue(not (q > q))
        self.assertTrue(not (q > Q))
        self.assertTrue(q > p)
        self.assertTrue(q > P)

    def testRepr(self):
        p = Foundation.NSPoint()
        self.assertEqual(repr(p), "<NSPoint x=0.0 y=0.0>")

        p = Foundation.NSPoint(42, 98)
        self.assertEqual(repr(p), "<NSPoint x=42 y=98>")

        p.x = p
        self.assertEqual(repr(p), "<NSPoint x=<NSPoint ...> y=98>")

    def testStr(self):
        p = Foundation.NSPoint()
        self.assertEqual(str(p), "<NSPoint x=0.0 y=0.0>")

        p = Foundation.NSPoint(42, 98)
        self.assertEqual(str(p), "<NSPoint x=42 y=98>")

        p.x = p
        self.assertEqual(repr(p), "<NSPoint x=<NSPoint ...> y=98>")

    def testSlice(self):
        p = Foundation.NSPoint(1, 2)
        q = p[:]

        self.assertIsInstance(q, tuple)
        self.assertEqual(q, (1.0, 2.0))

    def testDeleteAttr(self):
        p = Foundation.NSPoint(1, 2)
        self.assertRaises(TypeError, delattr, p, "x")

    def testDeleteSlice(self):
        p = Foundation.NSPoint(1, 2)
        self.assertRaises(TypeError, operator.delitem, p, 0)

    def testAssignSlice(self):
        p = Foundation.NSPoint(1, 2)
        p[:] = (4, 5)

        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 4)
        self.assertEqual(p.y, 5)

        p[:] = p
        self.assertIsInstance(p, Foundation.NSPoint)
        self.assertEqual(p.x, 4)
        self.assertEqual(p.y, 5)

        self.assertRaises(TypeError, do_set_slice, p, 0, 2, [1, 2, 3])
        self.assertRaises(TypeError, do_set_slice, p, 0, 2, [3])
        self.assertRaises(TypeError, do_set_slice, p, 0, 3, [1, 2, 3])

        self.assertRaises(TypeError, do_del_slice, p, 0, 0)
        self.assertRaises(TypeError, do_del_slice, p, 0, 1)
        self.assertRaises(TypeError, do_del_slice, p, 0, 2)


class TestNSSize(TestCase):
    def testConstructor(self):
        p = Foundation.NSSize()
        self.assertIsInstance(p, Foundation.NSSize)
        self.assertEqual(p.width, 0)
        self.assertEqual(p.height, 0)

        p = Foundation.NSSize(1, 2)
        self.assertIsInstance(p, Foundation.NSSize)
        self.assertEqual(p.width, 1)
        self.assertEqual(p.height, 2)
        self.assertEqual(p[0], 1)
        self.assertEqual(p[1], 2)

        p = Foundation.NSSize(height=1, width=2)
        self.assertIsInstance(p, Foundation.NSSize)
        self.assertEqual(p.width, 2)
        self.assertEqual(p.height, 1)
        self.assertEqual(p[1], 1)
        self.assertEqual(p[0], 2)

        self.assertRaises(TypeError, Foundation.NSSize, 1, 2, 3)
        self.assertRaises(TypeError, Foundation.NSSize, 1, 2, height=3)
        self.assertRaises(TypeError, Foundation.NSSize, 1, width=3)
        self.assertRaises(TypeError, Foundation.NSSize, width=3, z=4)

    def testMakeSize(self):
        p = Foundation.NSMakeSize(1, 2)
        self.assertIsInstance(p, Foundation.NSSize)
        self.assertEqual(p.width, 1)
        self.assertEqual(p.height, 2)


class TestNSRange(TestCase):
    def testConstructor(self):
        p = Foundation.NSRange()
        self.assertIsInstance(p, Foundation.NSRange)
        self.assertEqual(p.location, 0)
        self.assertEqual(p.length, 0)

        p = Foundation.NSRange(1, 2)
        self.assertIsInstance(p, Foundation.NSRange)
        self.assertEqual(p.location, 1)
        self.assertEqual(p.length, 2)
        self.assertEqual(p[0], 1)
        self.assertEqual(p[1], 2)

        p = Foundation.NSRange(length=1, location=2)
        self.assertIsInstance(p, Foundation.NSRange)
        self.assertEqual(p.location, 2)
        self.assertEqual(p.length, 1)
        self.assertEqual(p[1], 1)
        self.assertEqual(p[0], 2)

        self.assertRaises(TypeError, Foundation.NSRange, 1, 2, 3)
        self.assertRaises(TypeError, Foundation.NSRange, 1, 2, length=3)
        self.assertRaises(TypeError, Foundation.NSRange, 1, location=3)
        self.assertRaises(TypeError, Foundation.NSRange, location=3, z=4)

    def testMakeSize(self):
        p = Foundation.NSMakeSize(1, 2)
        self.assertIsInstance(p, Foundation.NSSize)
        self.assertEqual(p.width, 1)
        self.assertEqual(p.height, 2)


class TestNSRect(TestCase):
    def testConstructor(self):
        p = Foundation.NSRect()
        self.assertIsInstance(p, Foundation.NSRect)
        self.assertIsNot(p.origin, None)
        self.assertIsNot(p.size, None)
        self.assertEqual(p.origin, Foundation.NSPoint(0, 0))
        self.assertEqual(p.size, Foundation.NSSize(0, 0))

        p = Foundation.NSRect(1, 2)
        self.assertIsInstance(p, Foundation.NSRect)
        self.assertEqual(p.origin, 1)
        self.assertEqual(p.size, 2)
        self.assertEqual(p[0], 1)
        self.assertEqual(p[1], 2)

        p = Foundation.NSRect(size=1, origin=2)
        self.assertIsInstance(p, Foundation.NSRect)
        self.assertEqual(p.origin, 2)
        self.assertEqual(p.size, 1)
        self.assertEqual(p[1], 1)
        self.assertEqual(p[0], 2)

        self.assertRaises(TypeError, Foundation.NSRect, 1, 2, 3)
        self.assertRaises(TypeError, Foundation.NSRect, 1, 2, origin=3)
        self.assertRaises(TypeError, Foundation.NSRect, 1, origin=3)
        self.assertRaises(TypeError, Foundation.NSRect, origin=3, z=4)

    def testMakeRect(self):
        p = Foundation.NSMakeRect(1, 2, 3, 4)
        self.assertIsInstance(p, Foundation.NSRect)
        self.assertEqual(p.origin, (1, 2))
        self.assertEqual(p.size, (3, 4))
        self.assertEqual(p.origin.x, 1)
        self.assertEqual(p.origin.y, 2)
        self.assertEqual(p.size.width, 3)
        self.assertEqual(p.size.height, 4)

    def testNSEdgeInserts(self):
        v = Foundation.NSEdgeInsets()
        self.assertEqual(v.top, 0.0)
        self.assertEqual(v.left, 0.0)
        self.assertEqual(v.bottom, 0.0)
        self.assertEqual(v.right, 0.0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSScrubberHelper(AppKit.NSObject):
    def numberOfItemsForScrubber_(self, s):
        return 1

    def scrubber_viewForItemAtIndex_(self, s, i):
        return 1

    def scrubber_didSelectItemAtIndex_(self, s, i):
        return 1

    def scrubber_didHighlightItemAtIndex_(self, s, i):
        return 1

    def scrubber_didChangeVisibleRange_(self, s, i):
        return 1


class TestNSScrubber(TestCase):
    @min_os_level("10.12")
    def testConstants(self):
        self.assertEqual(AppKit.NSScrubberModeFixed, 0)
        self.assertEqual(AppKit.NSScrubberModeFree, 1)

        self.assertEqual(AppKit.NSScrubberAlignmentNone, 0)
        self.assertEqual(AppKit.NSScrubberAlignmentLeading, 1)
        self.assertEqual(AppKit.NSScrubberAlignmentTrailing, 2)
        self.assertEqual(AppKit.NSScrubberAlignmentCenter, 3)

    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultHasType(
            TestNSScrubberHelper.numberOfItemsForScrubber_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSScrubberHelper.scrubber_viewForItemAtIndex_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSScrubberHelper.scrubber_didSelectItemAtIndex_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSScrubberHelper.scrubber_didHighlightItemAtIndex_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSScrubberHelper.scrubber_didChangeVisibleRange_,
            1,
            AppKit.NSRange.__typestr__,
        )

        self.assertResultIsBOOL(AppKit.NSScrubber.isContinuous)
        self.assertArgIsBOOL(AppKit.NSScrubber.setContinuous_, 0)

        self.assertResultIsBOOL(AppKit.NSScrubber.floatsSelectionViews)
        self.assertArgIsBOOL(AppKit.NSScrubber.setFloatsSelectionViews_, 0)

        self.assertResultIsBOOL(AppKit.NSScrubber.showsArrowButtons)
        self.assertArgIsBOOL(AppKit.NSScrubber.setShowsArrowButtons_, 0)

        self.assertResultIsBOOL(AppKit.NSScrubber.showsAdditionalContentIndicators)
        self.assertArgIsBOOL(AppKit.NSScrubber.setShowsAdditionalContentIndicators_, 0)

        self.assertArgIsBlock(AppKit.NSScrubber.performSequentialBatchUpdates_, 0, b"v")

    @min_sdk_level("10.12")
    def testProtocols(self):
        objc.protocolNamed("NSScrubberDataSource")
        objc.protocolNamed("NSScrubberDelegate")



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {"NSMainNibFile": "ClassBrowser"}
setup(
    name="ClassBrowser",
    app=["ClassBrowser.py"],
    data_files=["ClassBrowser.nib"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



#
#  BookmarksDocument.py
#  Bookmarks
#
#  Converted by u.fiedler on 10.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Cocoa import NSDocument, NSKeyedArchiver, NSKeyedUnarchiver, NSMutableArray
from objc import super

# BookmarksDocument defines this as it may be used for copy and paste
# in addition to just drag and drop
CopiedRowsType = "COPIED_ROWS_TYPE"


class BookmarksDocument(NSDocument):
    bookmarksArray = objc.ivar("bookmarksArray")

    def init(self):
        self = super().init()
        if self is None:
            return None
        self.bookmarksArray = NSMutableArray.array()
        return self

    def windowNibName(self):
        return "BookmarksDocument"

    # Straightforward, standard document class
    # Allows content array to be saved, and file opened
    # Provides accessor methods for bookmarksArray

    # open and save -- very simple, just (un)archive bookmarksArray
    def dataRepresentationOfType_(self, aType):
        return NSKeyedArchiver.archivedDataWithRootObject_(self.bookmarksArray)

    def loadDataRepresentation_ofType_(self, data, aType):
        self.bookmarksArray = NSKeyedUnarchiver.unarchiveObjectWithData_(data)
        return True



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSScrubberItemView(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSScrubberArrangedView.isSelected)
        self.assertArgIsBOOL(AppKit.NSScrubberArrangedView.setSelected_, 0)

        self.assertResultIsBOOL(AppKit.NSScrubberArrangedView.isHighlighted)
        self.assertArgIsBOOL(AppKit.NSScrubberArrangedView.setHighlighted_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSTextViewportLayoutControllerHelper(AppKit.NSObject):
    def viewportBoundsForTextViewportLayoutController_(self, a):
        return 1


class TestNSTextViewportLayoutController(TestCase):
    @min_sdk_level("12.0")
    def test_protocols(self):
        objc.protocolNamed("NSTextViewportLayoutControllerDelegate")

    def test_methods(self):
        self.assertResultHasType(
            TestNSTextViewportLayoutControllerHelper.viewportBoundsForTextViewportLayoutController_,
            AppKit.NSRect.__typestr__,
        )



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSRegularExpression(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSRegularExpressionCaseInsensitive, 1 << 0)
        self.assertEqual(
            Foundation.NSRegularExpressionAllowCommentsAndWhitespace, 1 << 1
        )
        self.assertEqual(Foundation.NSRegularExpressionIgnoreMetacharacters, 1 << 2)
        self.assertEqual(Foundation.NSRegularExpressionDotMatchesLineSeparators, 1 << 3)
        self.assertEqual(Foundation.NSRegularExpressionAnchorsMatchLines, 1 << 4)
        self.assertEqual(Foundation.NSRegularExpressionUseUnixLineSeparators, 1 << 5)
        self.assertEqual(Foundation.NSRegularExpressionUseUnicodeWordBoundaries, 1 << 6)

        self.assertEqual(Foundation.NSMatchingReportProgress, 1 << 0)
        self.assertEqual(Foundation.NSMatchingReportCompletion, 1 << 1)
        self.assertEqual(Foundation.NSMatchingAnchored, 1 << 2)
        self.assertEqual(Foundation.NSMatchingWithTransparentBounds, 1 << 3)
        self.assertEqual(Foundation.NSMatchingWithoutAnchoringBounds, 1 << 4)

        self.assertEqual(Foundation.NSMatchingProgress, 1 << 0)
        self.assertEqual(Foundation.NSMatchingCompleted, 1 << 1)
        self.assertEqual(Foundation.NSMatchingHitEnd, 1 << 2)
        self.assertEqual(Foundation.NSMatchingRequiredEnd, 1 << 3)
        self.assertEqual(Foundation.NSMatchingInternalError, 1 << 4)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsOut(
            Foundation.NSRegularExpression.regularExpressionWithPattern_options_error_,
            2,
        )
        self.assertArgIsOut(
            Foundation.NSRegularExpression.initWithPattern_options_error_, 2
        )

        self.assertArgIsBlock(
            Foundation.NSRegularExpression.enumerateMatchesInString_options_range_usingBlock_,
            3,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsOut(Foundation.NSDataDetector.dataDetectorWithTypes_error_, 1)
        self.assertArgIsOut(Foundation.NSDataDetector.initWithTypes_error_, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMeasurement(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSMeasurement.canBeConvertedToUnit_)



import datetime
import decimal

import Cocoa
from objc._pythonify import OC_PythonFloat
from PyObjCTools import Conversion
from PyObjCTools.TestSupport import TestCase
import objc


class TestConversion(TestCase):
    def test_toPythonDecimal(self):
        v = Cocoa.NSDecimalNumber.decimalNumberWithString_("42.5")
        d = Conversion.toPythonDecimal(v)
        self.assertIsInstance(d, decimal.Decimal)
        self.assertEqual(str(d), "42.5")

    def test_fromPythonDecimal(self):
        d = decimal.Decimal("42.5")
        self.assertIsInstance(d, decimal.Decimal)

        v = Conversion.fromPythonDecimal(d)
        self.assertIsInstance(v, Cocoa.NSDecimalNumber)
        self.assertEqual(str(v), "42.5")

    def test_serializePropertyList(self):
        self.assertRaises(ValueError, Conversion.serializePropertyList, {}, "invalid")

        v = Conversion.serializePropertyList({"a": 42}, "xml")
        self.assertTrue(bytes(v).startswith(b"<?xml"))

        v = Conversion.serializePropertyList({"a": 42}, "binary")
        self.assertTrue(bytes(v).startswith(b"bplist"))

        self.assertRaises(
            ValueError, Conversion.serializePropertyList, {"a": 42}, "ascii"
        )

        self.assertRaises(
            ValueError,
            Conversion.serializePropertyList,
            Cocoa.NSObject.alloc().init(),
            "xml",
        )

    def test_deserializePropertyList(self):
        in_val = {"a": 42}

        for fmt in ("xml", "binary"):
            data = Conversion.serializePropertyList(in_val, fmt)

            out_val = Conversion.deserializePropertyList(data)

            self.assertEqual(in_val, out_val)

            bytes_data = bytes(data)
            out_val = Conversion.deserializePropertyList(bytes_data)
            self.assertEqual(in_val, out_val)

            if fmt == "xml":
                str_data = bytes_data.decode("utf-8")
                out_val = Conversion.deserializePropertyList(str_data)

                self.assertEqual(in_val, out_val)

            self.assertRaises(ValueError, Conversion.deserializePropertyList, data[:-2])

    def test_propertyListFromPythonCollection(self):
        for value, result_type in (
            ({"a": 42}, Cocoa.NSDictionary),
            ([42], Cocoa.NSArray),
            ((42,), Cocoa.NSArray),
            ({42}, Cocoa.NSSet),
            (frozenset({42}), Cocoa.NSSet),
            (datetime.datetime.now(), Cocoa.NSDate),
            (datetime.date.today(), Cocoa.NSDate),
        ):

            with self.subTest(value):
                v = Conversion.propertyListFromPythonCollection(value)
                self.assertIsInstance(v, result_type)
                self.assertEqual(v, value)

        with self.subTest("decimal"):
            v = Conversion.propertyListFromPythonCollection(decimal.Decimal(1))
            self.assertIsInstance(v, Cocoa.NSDecimalNumber)

        with self.subTest("invalid dict"):
            with self.assertRaises(TypeError):
                Conversion.propertyListFromPythonCollection({42: 43})

        with self.subTest("unknown type"):
            with self.assertRaises(TypeError):
                Conversion.propertyListFromPythonCollection(dir)

        with self.subTest("conversion helper"):

            def helper(v):
                return str(v)

            v = Conversion.propertyListFromPythonCollection(dir, helper)
            self.assertEqual(v, str(dir))

        with self.subTest("nested conversion"):
            value = [{"a": 42}, {"b": [1]}, {()}]

            v = Conversion.propertyListFromPythonCollection(value)

            self.assertIsInstance(v[0], Cocoa.NSDictionary)
            self.assertIsInstance(v[1], Cocoa.NSDictionary)
            self.assertIsInstance(v[1]["b"], Cocoa.NSArray)
            self.assertIsInstance(v[2], Cocoa.NSSet)
            self.assertIsInstance(next(iter(v[2])), Cocoa.NSArray)

    def test_pythonCollectionFromPropertyList(self):
        with self.subTest("dict"):
            value = Cocoa.NSDictionary.dictionaryWithDictionary_({"a": 42})
            self.assertIsInstance(value, Cocoa.NSDictionary)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, dict)
            self.assertEqual(v, {"a": 42})
            self.assertNotIsInstance(next(iter(v.keys())), objc.pyobjc_unicode)

        with self.subTest("list/tuple"):
            value = Cocoa.NSArray.arrayWithArray_(["a", 42])
            self.assertIsInstance(value, Cocoa.NSArray)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, list)
            self.assertEqual(v, ["a", 42])

        with self.subTest("set"):
            value = Cocoa.NSSet.setWithSet_({"a", 42})
            self.assertIsInstance(value, Cocoa.NSSet)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, set)
            self.assertEqual(v, {"a", 42})

        with self.subTest("bytes"):
            value = Cocoa.NSData.dataWithData_(b"hello")
            self.assertIsInstance(value, Cocoa.NSData)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, bytes)
            self.assertEqual(v, b"hello")

        with self.subTest("str"):
            value = Cocoa.NSString.stringWithString_("hello").nsstring()
            self.assertIsInstance(value, Cocoa.NSString)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, str)
            self.assertEqual(v, "hello")

        with self.subTest("float"):
            value = Cocoa.NSNumber.numberWithDouble_(1.5)
            self.assertIsInstance(value, Cocoa.NSNumber)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, float)
            self.assertNotIsInstance(v, OC_PythonFloat)
            self.assertEqual(v, 1.5)

        with self.subTest("int"):
            value = Cocoa.NSNumber.numberWithLong_(42)
            self.assertIsInstance(value, Cocoa.NSNumber)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, int)
            self.assertNotIsInstance(v, OC_PythonFloat)
            self.assertEqual(v, 42)

        with self.subTest("date"):
            value = Cocoa.NSDate.date()
            self.assertIsInstance(value, Cocoa.NSDate)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, datetime.datetime)

        with self.subTest("decimal"):
            value = Cocoa.NSDecimalNumber.decimalNumberWithString_("42.5")
            self.assertIsInstance(value, Cocoa.NSDecimalNumber)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIsInstance(v, decimal.Decimal)
            self.assertEqual(v, decimal.Decimal("42.5"))

        with self.subTest("None"):
            value = Cocoa.NSNull.null()
            self.assertIsInstance(value, Cocoa.NSNull)

            v = Conversion.pythonCollectionFromPropertyList(value)
            self.assertIs(v, None)

        with self.subTest("unknown type"):
            value = Cocoa.NSObject.alloc().init()

            with self.assertRaises(TypeError):
                Conversion.pythonCollectionFromPropertyList(value)

        with self.subTest("conversion helper"):

            def helper(value):
                return str(value)

            value = Cocoa.NSObject.alloc().init()

            v = Conversion.pythonCollectionFromPropertyList(value, helper)
            self.assertEqual(v, str(value))

        with self.subTest("nested"):
            value = Cocoa.NSMutableArray.array()
            value.append(Cocoa.NSArray.arrayWithArray_([1, 2]))
            value.append(
                Cocoa.NSDictionary.dictionaryWithDictionary_(
                    {42: Cocoa.NSArray.arrayWithArray_([4, 5])}
                )
            )
            value.append(
                Cocoa.NSSet.setWithSet_({Cocoa.NSArray.arrayWithArray_([4, 5])})
            )

            v = Conversion.pythonCollectionFromPropertyList(value)
            # self.assertEqual(v, value)

            self.assertIsInstance(v, list)
            self.assertEqual(len(v), 3)
            self.assertIsInstance(v[0], list)
            self.assertEqual(v[0], [1, 2])
            self.assertIsInstance(v[1], dict)
            self.assertIsInstance(v[1][42], list)
            self.assertEqual(v[1][42], [4, 5])
            self.assertIsInstance(v[2], set)
            self.assertIsInstance(next(iter(v[2])), tuple)
            self.assertEqual(next(iter(v[2])), (4, 5))



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSPointerArray(TestCase):
    def testPointers(self):
        o = Foundation.NSPointerArray.pointerArrayWithStrongObjects()

        m = o.addPointer_.__metadata__()
        self.assertEqual(m["arguments"][2]["type"], b"@")

        m = o.insertPointer_atIndex_.__metadata__()
        self.assertEqual(m["arguments"][2]["type"], b"@")

        m = o.replacePointerAtIndex_withPointer_.__metadata__()
        self.assertEqual(m["arguments"][3]["type"], b"@")

        m = o.pointerAtIndex_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"@")



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestPropertyList(TestCase):
    def testFunctions(self):
        dta = CoreFoundation.CFPropertyListCreateXMLData(None, {"key": 42, "key2": 1})
        self.assertIsInstance(dta, CoreFoundation.CFDataRef)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateFromXMLData, 3)
        v, err = CoreFoundation.CFPropertyListCreateFromXMLData(None, dta, 0, None)
        self.assertIs(err, None)
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        self.assertIn("key", v)
        self.assertIn("key2", v)
        self.assertEqual(v["key"], 42)
        self.assertEqual(v["key2"], True)
        v = CoreFoundation.CFPropertyListCreateDeepCopy(
            None, {"key": 42, "key2": True}, 0
        )
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        self.assertIn("key", v)
        self.assertIn("key2", v)
        self.assertEqual(v["key"], 42)
        self.assertEqual(v["key2"], True)
        valid = CoreFoundation.CFPropertyListIsValid(
            {"key": 42, "key2": True}, CoreFoundation.kCFPropertyListBinaryFormat_v1_0
        )
        self.assertIs(valid, True)

    def testStreams(self):
        stream = CoreFoundation.CFWriteStreamCreateWithAllocatedBuffers(
            CoreFoundation.kCFAllocatorDefault, CoreFoundation.kCFAllocatorDefault
        )
        r = CoreFoundation.CFWriteStreamOpen(stream)
        self.assertTrue(r)

        value = {"key1": 42, "key2": 1}

        self.assertArgIsOut(CoreFoundation.CFPropertyListWriteToStream, 3)
        rval, errorString = CoreFoundation.CFPropertyListWriteToStream(
            value, stream, CoreFoundation.kCFPropertyListXMLFormat_v1_0, None
        )
        self.assertIsInstance(rval, int)
        self.assertTrue(rval)
        self.assertIs(errorString, None)
        buf = CoreFoundation.CFWriteStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyDataWritten
        )
        self.assertIsInstance(buf, CoreFoundation.CFDataRef)
        buf = CoreFoundation.CFDataGetBytes(
            buf, (0, CoreFoundation.CFDataGetLength(buf)), None
        )
        self.assertIsInstance(buf, bytes)
        self.assertIn(b"<key>key1</key>", buf)
        self.assertIn(b"<integer>42</integer>", buf)
        self.assertIn(b"<key>key2</key>", buf)
        self.assertIn(b"<integer>1</integer>", buf)
        stream = CoreFoundation.CFReadStreamCreateWithBytesNoCopy(
            None, buf, len(buf), CoreFoundation.kCFAllocatorNull
        )
        r = CoreFoundation.CFReadStreamOpen(stream)
        self.assertTrue(r)

        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateFromStream, 4)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateFromStream, 5)
        res, plist_format, errorString = CoreFoundation.CFPropertyListCreateFromStream(
            None, stream, 0, 0, None, None
        )
        self.assertEqual(plist_format, CoreFoundation.kCFPropertyListXMLFormat_v1_0)
        self.assertIs(errorString, None)
        self.assertEqual(res, value)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFPropertyListImmutable, 0)
        self.assertEqual(CoreFoundation.kCFPropertyListMutableContainers, 1 << 0)
        self.assertEqual(
            CoreFoundation.kCFPropertyListMutableContainersAndLeaves, 1 << 1
        )

        self.assertEqual(CoreFoundation.kCFPropertyListOpenStepFormat, 1)
        self.assertEqual(CoreFoundation.kCFPropertyListXMLFormat_v1_0, 100)
        self.assertEqual(CoreFoundation.kCFPropertyListBinaryFormat_v1_0, 200)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreFoundation.kCFPropertyListReadCorruptError, 3840)
        self.assertEqual(CoreFoundation.kCFPropertyListReadUnknownVersionError, 3841)
        self.assertEqual(CoreFoundation.kCFPropertyListReadStreamError, 3842)
        self.assertEqual(CoreFoundation.kCFPropertyListWriteStreamError, 3851)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        dta = CoreFoundation.CFPropertyListCreateXMLData(None, {"key": 42, "key2": 1})
        self.assertIsInstance(dta, CoreFoundation.CFDataRef)
        bytes_value = CoreFoundation.CFDataGetBytes(
            dta, (0, CoreFoundation.CFDataGetLength(dta)), None
        )
        self.assertIsNot(bytes_value, None)
        self.assertResultIsCFRetained(CoreFoundation.CFPropertyListCreateWithData)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateWithData, 3)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateWithData, 4)
        v, fmt, err = CoreFoundation.CFPropertyListCreateWithData(
            None, dta, 0, None, None
        )
        self.assertIsNot(v, None)
        self.assertIsInstance(fmt, int)
        self.assertIs(err, None)
        stream = CoreFoundation.CFReadStreamCreateWithBytesNoCopy(
            None, bytes_value, len(bytes_value), CoreFoundation.kCFAllocatorNull
        )
        CoreFoundation.CFReadStreamOpen(stream)

        self.assertResultIsCFRetained(CoreFoundation.CFPropertyListCreateWithStream)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateWithStream, 4)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateWithStream, 5)
        v, fmt, err = CoreFoundation.CFPropertyListCreateWithStream(
            None, stream, len(bytes_value), 0, None, None
        )
        self.assertIsNot(v, None)
        self.assertIsInstance(fmt, int)
        self.assertIs(err, None)
        import array

        buf = array.array("b", b" " * 1024)

        stream = CoreFoundation.CFWriteStreamCreateWithBuffer(None, buf, 1024)
        CoreFoundation.CFWriteStreamOpen(stream)

        self.assertArgIsOut(CoreFoundation.CFPropertyListWrite, 4)
        cnt, err = CoreFoundation.CFPropertyListWrite(
            {"key": 42},
            stream,
            CoreFoundation.kCFPropertyListBinaryFormat_v1_0,
            0,
            None,
        )
        self.assertNotEqual(cnt, 0)
        self.assertEqual(err, None)

        self.assertResultIsCFRetained(CoreFoundation.CFPropertyListCreateData)
        self.assertArgIsOut(CoreFoundation.CFPropertyListCreateData, 4)
        dta, err = CoreFoundation.CFPropertyListCreateData(
            None,
            {"key": "value"},
            CoreFoundation.kCFPropertyListBinaryFormat_v1_0,
            0,
            None,
        )
        self.assertIsInstance(dta, CoreFoundation.CFDataRef)
        self.assertEqual(err, None)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSGridView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSGridCellPlacementInherited, 0)
        self.assertEqual(AppKit.NSGridCellPlacementNone, 1)
        self.assertEqual(AppKit.NSGridCellPlacementLeading, 2)
        self.assertEqual(AppKit.NSGridCellPlacementTop, 2)
        self.assertEqual(AppKit.NSGridCellPlacementTrailing, 3)
        self.assertEqual(AppKit.NSGridCellPlacementBottom, 3)
        self.assertEqual(AppKit.NSGridCellPlacementCenter, 4)
        self.assertEqual(AppKit.NSGridCellPlacementFill, 5)

        self.assertEqual(AppKit.NSGridRowAlignmentInherited, 0)
        self.assertEqual(AppKit.NSGridRowAlignmentNone, 1)
        self.assertEqual(AppKit.NSGridRowAlignmentFirstBaseline, 2)
        self.assertEqual(AppKit.NSGridRowAlignmentLastBaseline, 3)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSGridViewSizeForContent, float)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSGridRow.isHidden)
        self.assertArgIsBOOL(AppKit.NSGridRow.setHidden_, 0)

        self.assertResultIsBOOL(AppKit.NSGridColumn.isHidden)
        self.assertArgIsBOOL(AppKit.NSGridColumn.setHidden_, 0)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPDFPanel(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSPDFPanelShowsPaperSize, 1 << 2)
        self.assertEqual(AppKit.NSPDFPanelShowsOrientation, 1 << 3)
        self.assertEqual(AppKit.NSPDFPanelRequestsParentDirectory, 1 << 24)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertArgIsBlock(
            AppKit.NSPDFPanel.beginSheetWithPDFInfo_modalForWindow_completionHandler_,
            2,
            objc._C_VOID + objc._C_NSInteger,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSCustomImageRep(TestCase):
    def testMethods(self):
        self.assertArgIsSEL(
            AppKit.NSCustomImageRep.initWithDrawSelector_delegate_, 0, b"v@:@"
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBOOL(
            AppKit.NSCustomImageRep.initWithSize_flipped_drawingHandler_, 1
        )
        self.assertArgIsBlock(
            AppKit.NSCustomImageRep.initWithSize_flipped_drawingHandler_,
            2,
            objc._C_NSBOOL + AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBlock(
            AppKit.NSCustomImageRep.drawingHandler,
            objc._C_NSBOOL + AppKit.NSRect.__typestr__,
        )



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSGraphics(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSCompositeClear, 0)
        self.assertEqual(AppKit.NSCompositeCopy, 1)
        self.assertEqual(AppKit.NSCompositeSourceOver, 2)
        self.assertEqual(AppKit.NSCompositeSourceIn, 3)
        self.assertEqual(AppKit.NSCompositeSourceOut, 4)
        self.assertEqual(AppKit.NSCompositeSourceAtop, 5)
        self.assertEqual(AppKit.NSCompositeDestinationOver, 6)
        self.assertEqual(AppKit.NSCompositeDestinationIn, 7)
        self.assertEqual(AppKit.NSCompositeDestinationOut, 8)
        self.assertEqual(AppKit.NSCompositeDestinationAtop, 9)
        self.assertEqual(AppKit.NSCompositeXOR, 10)
        self.assertEqual(AppKit.NSCompositePlusDarker, 11)
        self.assertEqual(AppKit.NSCompositeHighlight, 12)
        self.assertEqual(AppKit.NSCompositePlusLighter, 13)
        self.assertEqual(AppKit.NSCompositeMultiply, 14)
        self.assertEqual(AppKit.NSCompositeScreen, 15)
        self.assertEqual(AppKit.NSCompositeOverlay, 16)
        self.assertEqual(AppKit.NSCompositeDarken, 17)
        self.assertEqual(AppKit.NSCompositeLighten, 18)
        self.assertEqual(AppKit.NSCompositeColorDodge, 19)
        self.assertEqual(AppKit.NSCompositeColorBurn, 20)
        self.assertEqual(AppKit.NSCompositeSoftLight, 21)
        self.assertEqual(AppKit.NSCompositeHardLight, 22)
        self.assertEqual(AppKit.NSCompositeDifference, 23)
        self.assertEqual(AppKit.NSCompositeExclusion, 24)
        self.assertEqual(AppKit.NSCompositeHue, 25)
        self.assertEqual(AppKit.NSCompositeSaturation, 26)
        self.assertEqual(AppKit.NSCompositeColor, 27)
        self.assertEqual(AppKit.NSCompositeLuminosity, 28)

        self.assertEqual(AppKit.NSCompositingOperationClear, 0)
        self.assertEqual(AppKit.NSCompositingOperationCopy, 1)
        self.assertEqual(AppKit.NSCompositingOperationSourceOver, 2)
        self.assertEqual(AppKit.NSCompositingOperationSourceIn, 3)
        self.assertEqual(AppKit.NSCompositingOperationSourceOut, 4)
        self.assertEqual(AppKit.NSCompositingOperationSourceAtop, 5)
        self.assertEqual(AppKit.NSCompositingOperationDestinationOver, 6)
        self.assertEqual(AppKit.NSCompositingOperationDestinationIn, 7)
        self.assertEqual(AppKit.NSCompositingOperationDestinationOut, 8)
        self.assertEqual(AppKit.NSCompositingOperationDestinationAtop, 9)
        self.assertEqual(AppKit.NSCompositingOperationXOR, 10)
        self.assertEqual(AppKit.NSCompositingOperationPlusDarker, 11)
        self.assertEqual(AppKit.NSCompositingOperationHighlight, 12)
        self.assertEqual(AppKit.NSCompositingOperationPlusLighter, 13)
        self.assertEqual(AppKit.NSCompositingOperationMultiply, 14)
        self.assertEqual(AppKit.NSCompositingOperationScreen, 15)
        self.assertEqual(AppKit.NSCompositingOperationOverlay, 16)
        self.assertEqual(AppKit.NSCompositingOperationDarken, 17)
        self.assertEqual(AppKit.NSCompositingOperationLighten, 18)
        self.assertEqual(AppKit.NSCompositingOperationColorDodge, 19)
        self.assertEqual(AppKit.NSCompositingOperationColorBurn, 20)
        self.assertEqual(AppKit.NSCompositingOperationSoftLight, 21)
        self.assertEqual(AppKit.NSCompositingOperationHardLight, 22)
        self.assertEqual(AppKit.NSCompositingOperationDifference, 23)
        self.assertEqual(AppKit.NSCompositingOperationExclusion, 24)
        self.assertEqual(AppKit.NSCompositingOperationHue, 25)
        self.assertEqual(AppKit.NSCompositingOperationSaturation, 26)
        self.assertEqual(AppKit.NSCompositingOperationColor, 27)
        self.assertEqual(AppKit.NSCompositingOperationLuminosity, 28)

        self.assertEqual(AppKit.NSBackingStoreRetained, 0)
        self.assertEqual(AppKit.NSBackingStoreNonretained, 1)
        self.assertEqual(AppKit.NSBackingStoreBuffered, 2)

        self.assertEqual(AppKit.NSWindowAbove, 1)
        self.assertEqual(AppKit.NSWindowBelow, -1)
        self.assertEqual(AppKit.NSWindowOut, 0)

        self.assertEqual(AppKit.NSFocusRingOnly, 0)
        self.assertEqual(AppKit.NSFocusRingBelow, 1)
        self.assertEqual(AppKit.NSFocusRingAbove, 2)

        self.assertEqual(AppKit.NSFocusRingTypeDefault, 0)
        self.assertEqual(AppKit.NSFocusRingTypeNone, 1)
        self.assertEqual(AppKit.NSFocusRingTypeExterior, 2)

        self.assertIsInstance(AppKit.NSCalibratedWhiteColorSpace, str)
        self.assertIsInstance(AppKit.NSCalibratedBlackColorSpace, str)
        self.assertIsInstance(AppKit.NSCalibratedRGBColorSpace, str)
        self.assertIsInstance(AppKit.NSDeviceWhiteColorSpace, str)
        self.assertIsInstance(AppKit.NSDeviceBlackColorSpace, str)
        self.assertIsInstance(AppKit.NSDeviceRGBColorSpace, str)
        self.assertIsInstance(AppKit.NSDeviceCMYKColorSpace, str)
        self.assertIsInstance(AppKit.NSNamedColorSpace, str)
        self.assertIsInstance(AppKit.NSPatternColorSpace, str)
        self.assertIsInstance(AppKit.NSCustomColorSpace, str)
        self.assertIsInstance(AppKit.NSWhite, float)
        self.assertIsInstance(AppKit.NSLightGray, float)
        self.assertIsInstance(AppKit.NSDarkGray, float)
        self.assertIsInstance(AppKit.NSBlack, float)

        self.assertIsInstance(AppKit.NSDeviceResolution, str)
        self.assertIsInstance(AppKit.NSDeviceColorSpaceName, str)
        self.assertIsInstance(AppKit.NSDeviceBitsPerSample, str)
        self.assertIsInstance(AppKit.NSDeviceIsScreen, str)
        self.assertIsInstance(AppKit.NSDeviceIsPrinter, str)
        self.assertIsInstance(AppKit.NSDeviceSize, str)
        self.assertEqual(AppKit.NSAnimationEffectDisappearingItemDefault, 0)
        self.assertEqual(AppKit.NSAnimationEffectPoof, 10)

        self.assertEqual(AppKit.NSDisplayGamutSRGB, 1)
        self.assertEqual(AppKit.NSDisplayGamutP3, 2)

    def testFunctions(self):
        app = AppKit.NSApplication.sharedApplication()  # noqa: F841

        self.assertArgHasType(AppKit.NSBestDepth, 4, b"o^" + objc._C_NSBOOL)
        self.assertArgIsBOOL(AppKit.NSBestDepth, 3)
        d, e = AppKit.NSBestDepth(AppKit.NSDeviceRGBColorSpace, 8, 32, False, None)
        self.assertIsInstance(d, int)
        self.assertIsInstance(e, bool)

        self.assertResultIsBOOL(AppKit.NSPlanarFromDepth)
        self.assertIsInstance(AppKit.NSPlanarFromDepth(0), bool)

        self.assertIsInstance(AppKit.NSColorSpaceFromDepth(0), str)
        self.assertIsInstance(AppKit.NSBitsPerSampleFromDepth(0), int)
        self.assertIsInstance(AppKit.NSBitsPerPixelFromDepth(0), int)
        self.assertIsInstance(
            AppKit.NSNumberOfColorComponents(AppKit.NSDeviceRGBColorSpace), int
        )

        v = AppKit.NSAvailableWindowDepths()
        self.assertIsInstance(v, tuple)
        self.assertNotEqual(len(v), 0)
        self.assertIsInstance(v[0], int)

        img = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bitmapFormat_bytesPerRow_bitsPerPixel_(  # noqa: B950
            None, 255, 255, 8, 4, True, False, AppKit.NSCalibratedRGBColorSpace, 0, 0, 0
        )

        context = AppKit.NSGraphicsContext.graphicsContextWithBitmapImageRep_(img)
        current = AppKit.NSGraphicsContext.currentContext()
        try:
            AppKit.NSGraphicsContext.setCurrentContext_(context)
            AppKit.NSRectFill(((0, 0), (1, 2)))

            self.assertArgSizeInArg(AppKit.NSRectFillList, 0, 1)
            AppKit.NSRectFillList([((0, 0), (1, 2)), ((10, 50), (9, 9))], 2)

            self.assertArgSizeInArg(AppKit.NSRectFillListWithGrays, 0, 2)
            self.assertArgSizeInArg(AppKit.NSRectFillListWithGrays, 1, 2)
            AppKit.NSRectFillListWithGrays(
                [((0, 0), (1, 2)), ((10, 50), (9, 9))], (0.5, 0.6), 2
            )

            self.assertArgSizeInArg(AppKit.NSRectFillListWithColors, 0, 2)
            self.assertArgSizeInArg(AppKit.NSRectFillListWithColors, 1, 2)
            AppKit.NSRectFillListWithColors(
                [((0, 0), (1, 2)), ((10, 50), (9, 9))],
                (AppKit.NSColor.blueColor(), AppKit.NSColor.redColor()),
                2,
            )

            AppKit.NSRectFillUsingOperation(
                ((0, 0), (1, 2)), AppKit.NSCompositeSourceOver
            )

            self.assertArgSizeInArg(AppKit.NSRectFillListUsingOperation, 0, 1)
            AppKit.NSRectFillListUsingOperation(
                [((0, 0), (1, 2)), ((10, 50), (9, 9))], 2, AppKit.NSCompositeSourceOver
            )

            self.assertArgSizeInArg(AppKit.NSRectFillListWithColorsUsingOperation, 0, 2)
            self.assertArgSizeInArg(AppKit.NSRectFillListWithColorsUsingOperation, 1, 2)
            AppKit.NSRectFillListWithColorsUsingOperation(
                [((0, 0), (1, 2)), ((10, 50), (9, 9))],
                (AppKit.NSColor.blueColor(), AppKit.NSColor.redColor()),
                2,
                AppKit.NSCompositeSourceOver,
            )

            AppKit.NSFrameRect(((5, 5), (20, 30)))
            AppKit.NSFrameRectWithWidth(((5, 5), (20, 30)), 4)
            AppKit.NSFrameRectWithWidthUsingOperation(
                ((5, 5), (20, 30)), 4, AppKit.NSCompositeSourceOver
            )

            AppKit.NSRectClip(((5, 5), (200, 200)))
            self.assertArgSizeInArg(AppKit.NSRectClipList, 0, 1)
            AppKit.NSRectClipList([((5, 5), (200, 200)), ((50, 50), (90, 100))], 2)

            color = AppKit.NSReadPixel((5, 5))
            self.assertIsInstance(color, AppKit.NSColor)

            self.assertArgSizeInArg(AppKit.NSDrawTiledRects, 2, 4)
            self.assertArgSizeInArg(AppKit.NSDrawTiledRects, 3, 4)
            self.assertArgIsIn(AppKit.NSDrawTiledRects, 2)
            self.assertArgIsIn(AppKit.NSDrawTiledRects, 3)
            AppKit.NSDrawTiledRects(
                ((10, 10), (50, 50)),
                ((15, 15), (10, 10)),
                [AppKit.NSMinXEdge, AppKit.NSMaxXEdge],
                [0.8, 0.9],
                2,
            )

            AppKit.NSDrawGrayBezel(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSDrawGroove(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSDrawWhiteBezel(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSDrawButton(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSEraseRect(((0, 0), (10, 10)))
            AppKit.NSCopyBits(0, ((10, 10), (50, 50)), (50, 50))
            AppKit.NSHighlightRect(((10, 10), (50, 50)))
            AppKit.NSDrawDarkBezel(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSDrawLightBezel(((0, 0), (10, 10)), ((0, 0), (50, 50)))
            AppKit.NSDottedFrameRect(((10, 10), (50, 50)))
            AppKit.NSDrawWindowBackground(((10, 10), (50, 50)))

        finally:
            AppKit.NSGraphicsContext.setCurrentContext_(current)

            AppKit.NSSetFocusRingStyle(AppKit.NSFocusRingAbove)

            self.assertArgIsOut(AppKit.NSGetWindowServerMemory, 1)
            self.assertArgIsOut(AppKit.NSGetWindowServerMemory, 2)
            self.assertArgIsOut(AppKit.NSGetWindowServerMemory, 3)
            r = AppKit.NSGetWindowServerMemory(0, None, None, None)
            self.assertIsInstance(r[0], int)
            self.assertIsInstance(r[1], int)
            self.assertIsInstance(r[2], int)

            self.assertArgSizeInArg(AppKit.NSDrawColorTiledRects, 2, 4)
            self.assertArgSizeInArg(AppKit.NSDrawColorTiledRects, 3, 4)
            self.assertArgIsIn(AppKit.NSDrawColorTiledRects, 2)
            self.assertArgIsIn(AppKit.NSDrawColorTiledRects, 3)
            AppKit.NSDrawColorTiledRects(
                ((10, 10), (50, 50)),
                ((15, 15), (10, 10)),
                [AppKit.NSMinXEdge, AppKit.NSMaxXEdge],
                [AppKit.NSColor.redColor(), AppKit.NSColor.blueColor()],
                2,
            )

            # self.assertArgIsBOOL(AppKit.NSDrawBitmap, 7)
            # self.assertArgIsBOOL(AppKit.NSDrawBitmap, 8)
            # AppKit.NSDrawBitmap(((0, 0), (10, 10)), 10, 20, 8, 4, 32, 40, False, True,
            #        AppKit.NSDeviceRGBColorSpace, [' '*4*10*20, '', '', '', ''])

            self.assertArgSizeInArg(AppKit.NSWindowList, 1, 0)
            self.assertArgIsOut(AppKit.NSWindowList, 1)
            v = AppKit.NSWindowList(5, None)
            self.assertIsInstance(v, tuple)
            self.assertEqual(len(v), 5)
            self.assertIsInstance(v[0], int)

            self.assertArgIsOut(AppKit.NSCountWindowsForContext, 1)
            v = AppKit.NSCountWindowsForContext(1, None)
            self.assertIsInstance(v, int)

            self.assertArgIsOut(AppKit.NSWindowListForContext, 2)
            self.assertArgSizeInArg(AppKit.NSWindowListForContext, 2, 1)
            v = AppKit.NSWindowListForContext(0, 5, None)
            self.assertIsInstance(v, tuple)
            self.assertEqual(len(v), 5)
            self.assertIsInstance(v[0], int)

        del img
        del context
        AppKit.NSBeep()
        count = AppKit.NSCountWindows(None)
        self.assertIsInstance(count, int)

        try:
            AppKit.NSDisableScreenUpdates()
        except objc.error:
            pass

        try:
            AppKit.NSEnableScreenUpdates()
        except objc.error:
            pass
        self.assertArgIsSEL(AppKit.NSShowAnimationEffect, 4, b"v@:^v")
        self.assertArgHasType(AppKit.NSShowAnimationEffect, 5, b"^v")
        try:
            AppKit.NSShowAnimationEffect(
                AppKit.NSAnimationEffectPoof, (10, 10), (20, 30), None, None, None
            )
        except objc.error:
            pass

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSColorRenderingIntentDefault, 0)
        self.assertEqual(AppKit.NSColorRenderingIntentAbsoluteColorimetric, 1)
        self.assertEqual(AppKit.NSColorRenderingIntentRelativeColorimetric, 2)
        self.assertEqual(AppKit.NSColorRenderingIntentPerceptual, 3)
        self.assertEqual(AppKit.NSColorRenderingIntentSaturation, 4)

        self.assertEqual(AppKit.NSImageInterpolationDefault, 0)
        self.assertEqual(AppKit.NSImageInterpolationNone, 1)
        self.assertEqual(AppKit.NSImageInterpolationLow, 2)
        self.assertEqual(AppKit.NSImageInterpolationHigh, 3)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSWindowDepthTwentyfourBitRGB, 0x208)
        self.assertEqual(AppKit.NSWindowDepthSixtyfourBitRGB, 0x210)
        self.assertEqual(AppKit.NSWindowDepthOnehundredtwentyeightBitRGB, 0x220)

        self.assertEqual(AppKit.NSImageInterpolationMedium, 4)

    AppKit.NSApplication.sharedApplication()



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSScroller(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSScrollerArrowsMaxEnd, 0)
        self.assertEqual(AppKit.NSScrollerArrowsMinEnd, 1)
        self.assertEqual(AppKit.NSScrollerArrowsDefaultSetting, 0)
        self.assertEqual(AppKit.NSScrollerArrowsNone, 2)

        self.assertEqual(AppKit.NSNoScrollerParts, 0)
        self.assertEqual(AppKit.NSOnlyScrollerArrows, 1)
        self.assertEqual(AppKit.NSAllScrollerParts, 2)

        self.assertEqual(AppKit.NSScrollerNoPart, 0)
        self.assertEqual(AppKit.NSScrollerDecrementPage, 1)
        self.assertEqual(AppKit.NSScrollerKnob, 2)
        self.assertEqual(AppKit.NSScrollerIncrementPage, 3)
        self.assertEqual(AppKit.NSScrollerDecrementLine, 4)
        self.assertEqual(AppKit.NSScrollerIncrementLine, 5)
        self.assertEqual(AppKit.NSScrollerKnobSlot, 6)

        self.assertEqual(AppKit.NSScrollerIncrementArrow, 0)
        self.assertEqual(AppKit.NSScrollerDecrementArrow, 1)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSScrollerStyleLegacy, 0)
        self.assertEqual(AppKit.NSScrollerStyleOverlay, 1)

        self.assertEqual(AppKit.NSScrollerKnobStyleDefault, 0)
        self.assertEqual(AppKit.NSScrollerKnobStyleDark, 1)
        self.assertEqual(AppKit.NSScrollerKnobStyleLight, 2)

        self.assertIsInstance(AppKit.NSPreferredScrollerStyleDidChangeNotification, str)

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSScroller.drawArrow_highlight_, 1)
        self.assertArgIsBOOL(AppKit.NSScroller.drawKnobSlotInRect_highlight_, 1)
        self.assertArgIsBOOL(AppKit.NSScroller.highlight_, 0)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSScroller.isCompatibleWithOverlayScrollers)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "CFBundleIdentifier": "net.sf.pyobjc.TinyURLService",
    "LSBackgroundOnly": 1,
    "NSServices": [
        {
            "NSKeyEquivalent": {"default": "0"},
            "NSMenuItem": {"default": "Shorten URL"},
            "NSMessage": "doTinyURLService",
            "NSPortName": "TinyURLService",
            "NSReturnTypes": ["NSStringPboardType"],
            "NSSendTypes": ["NSStringPboardType"],
        }
    ],
}


setup(
    app=["TinyURLService.py"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestCFDictionary(TestCase):
    def testCreation(self):
        dictionary = CoreFoundation.CFDictionaryCreate(
            None,
            ("aap", "noot", "mies", "wim"),
            ("monkey", "nut", "missy", "john"),
            4,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertIsInstance(dictionary, CoreFoundation.CFDictionaryRef)
        self.assertEqual(
            dictionary, {"aap": "monkey", "noot": "nut", "mies": "missy", "wim": "john"}
        )

        dictionary = CoreFoundation.CFDictionaryCreateMutable(
            None,
            0,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertIsInstance(dictionary, CoreFoundation.CFMutableDictionaryRef)
        CoreFoundation.CFDictionarySetValue(dictionary, "hello", "world")
        self.assertEqual(dictionary, {"hello": "world"})

    def testApplyFunction(self):
        dictionary = CoreFoundation.CFDictionaryCreate(
            None,
            ("aap", "noot", "mies", "wim"),
            ("monkey", "nut", "missy", "john"),
            4,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )

        context = []

        def function(key, value, context):
            context.append((key, value))

        self.assertArgIsFunction(
            CoreFoundation.CFDictionaryApplyFunction, 1, b"v@@@", False
        )
        self.assertArgHasType(CoreFoundation.CFDictionaryApplyFunction, 2, b"@")
        CoreFoundation.CFDictionaryApplyFunction(dictionary, function, context)

        context.sort()
        self.assertEqual(len(context), 4)
        self.assertEqual(
            context,
            [("aap", "monkey"), ("mies", "missy"), ("noot", "nut"), ("wim", "john")],
        )

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFDictionaryGetTypeID(), int)

    def testCreation2(self):  # XXX
        dct = CoreFoundation.CFDictionaryCreate(
            None,
            ["key1" "key2"],
            [42, 43],
            2,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertIsInstance(dct, CoreFoundation.CFDictionaryRef)
        dct = CoreFoundation.CFDictionaryCreateCopy(None, dct)
        self.assertIsInstance(dct, CoreFoundation.CFDictionaryRef)
        dct = CoreFoundation.CFDictionaryCreateMutable(
            None,
            0,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertIsInstance(dct, CoreFoundation.CFDictionaryRef)
        dct = CoreFoundation.CFDictionaryCreateMutableCopy(None, 0, dct)
        self.assertIsInstance(dct, CoreFoundation.CFDictionaryRef)

    def testInspection(self):
        dct = CoreFoundation.CFDictionaryCreate(
            None,
            ["key1", "key2"],
            [42, 42],
            2,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertIsInstance(dct, CoreFoundation.CFDictionaryRef)
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 2)
        self.assertEqual(CoreFoundation.CFDictionaryGetCountOfKey(dct, "key1"), 1)
        self.assertEqual(CoreFoundation.CFDictionaryGetCountOfKey(dct, "key3"), 0)
        self.assertEqual(CoreFoundation.CFDictionaryGetCountOfValue(dct, 42), 2)
        self.assertEqual(CoreFoundation.CFDictionaryGetCountOfValue(dct, 44), 0)
        self.assertResultHasType(CoreFoundation.CFDictionaryContainsKey, objc._C_NSBOOL)
        self.assertTrue(CoreFoundation.CFDictionaryContainsKey(dct, "key1"))
        self.assertFalse(CoreFoundation.CFDictionaryContainsKey(dct, "key3"))

        self.assertResultHasType(
            CoreFoundation.CFDictionaryContainsValue, objc._C_NSBOOL
        )
        self.assertTrue(CoreFoundation.CFDictionaryContainsValue(dct, 42))
        self.assertFalse(CoreFoundation.CFDictionaryContainsValue(dct, "key3"))

        self.assertEqual(CoreFoundation.CFDictionaryGetValue(dct, "key2"), 42)
        self.assertIs(CoreFoundation.CFDictionaryGetValue(dct, "key3"), None)
        self.assertResultHasType(
            CoreFoundation.CFDictionaryGetValueIfPresent, objc._C_NSBOOL
        )
        self.assertArgIsOut(CoreFoundation.CFDictionaryGetValueIfPresent, 2)
        ok, value = CoreFoundation.CFDictionaryGetValueIfPresent(dct, "key2", None)
        self.assertTrue(ok)
        self.assertEqual(value, 42)
        ok, value = CoreFoundation.CFDictionaryGetValueIfPresent(dct, "key3", None)
        self.assertFalse(ok)
        self.assertIs(value, None)
        keys, values = CoreFoundation.CFDictionaryGetKeysAndValues(dct, None, None)
        self.assertEqual(values, (42, 42))
        keys = list(keys)
        keys.sort()
        self.assertEqual(keys, ["key1", "key2"])

    def testMutation(self):
        dct = CoreFoundation.CFDictionaryCreateMutable(
            None,
            0,
            CoreFoundation.kCFTypeDictionaryKeyCallBacks,
            CoreFoundation.kCFTypeDictionaryValueCallBacks,
        )
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 0)
        CoreFoundation.CFDictionaryAddValue(dct, "key1", "value1")
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 1)
        self.assertTrue(CoreFoundation.CFDictionaryContainsKey(dct, "key1"))

        CoreFoundation.CFDictionarySetValue(dct, "key2", "value2")
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 2)
        self.assertTrue(CoreFoundation.CFDictionaryContainsKey(dct, "key2"))

        CoreFoundation.CFDictionaryReplaceValue(dct, "key2", "value2b")
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 2)
        self.assertTrue(CoreFoundation.CFDictionaryContainsKey(dct, "key2"))
        self.assertEqual(CoreFoundation.CFDictionaryGetValue(dct, "key2"), "value2b")
        CoreFoundation.CFDictionaryReplaceValue(dct, "key3", "value2b")
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 2)
        self.assertFalse(CoreFoundation.CFDictionaryContainsKey(dct, "key3"))

        CoreFoundation.CFDictionaryRemoveValue(dct, "key1")
        self.assertFalse(CoreFoundation.CFDictionaryContainsKey(dct, "key1"))

        CoreFoundation.CFDictionaryRemoveAllValues(dct)
        self.assertFalse(CoreFoundation.CFDictionaryContainsKey(dct, "key2"))
        self.assertEqual(CoreFoundation.CFDictionaryGetCount(dct), 0)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTextLayoutFragment(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSTextLayoutFragmentEnumerationOptionsNone, 0)
        self.assertEqual(AppKit.NSTextLayoutFragmentEnumerationOptionsReverse, 1 << 0)
        self.assertEqual(
            AppKit.NSTextLayoutFragmentEnumerationOptionsEstimatesSize, 1 << 1
        )
        self.assertEqual(
            AppKit.NSTextLayoutFragmentEnumerationOptionsEnsuresLayout, 1 << 2
        )
        self.assertEqual(
            AppKit.NSTextLayoutFragmentEnumerationOptionsEnsuresExtraLineFragment,
            1 << 3,
        )

        self.assertEqual(AppKit.NSTextLayoutFragmentStateNone, 0)
        self.assertEqual(AppKit.NSTextLayoutFragmentStateEstimatedUsageBounds, 1)
        self.assertEqual(AppKit.NSTextLayoutFragmentStateCalculatedUsageBounds, 2)
        self.assertEqual(AppKit.NSTextLayoutFragmentStateLayoutAvailable, 3)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSLevelIndicator(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSLevelIndicatorPlaceholderVisibilityAutomatic, 0)
        self.assertEqual(AppKit.NSLevelIndicatorPlaceholderVisibilityAlways, 1)
        self.assertEqual(AppKit.NSLevelIndicatorPlaceholderVisibilityWhileEditing, 2)

    @min_os_level("10.13")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSLevelIndicator.drawsTieredCapacityLevels)
        self.assertArgIsBOOL(AppKit.NSLevelIndicator.setDrawsTieredCapacityLevels_, 0)

        self.assertResultIsBOOL(AppKit.NSLevelIndicator.isEditable)
        self.assertArgIsBOOL(AppKit.NSLevelIndicator.setEditable_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class Presenter(Foundation.NSObject):
    def relinquishPresentedItemToReader_(self, a):
        pass

    def relinquishPresentedItemToWriter_(self, a):
        pass

    def savePresentedItemChangesWithCompletionHandler_(self, a):
        pass

    def accommodatePresentedItemDeletionWithCompletionHandler_(self, a):
        pass

    def accommodatePresentedSubitemDeletionAtURL_completionHandler_(self, a, b):
        pass


class TestNSFilePresenter(TestCase):
    @min_os_level("10.7")
    def testProtocols(self):
        objc.protocolNamed("NSFilePresenter")

        self.assertArgIsBlock(
            Presenter.relinquishPresentedItemToReader_, 0, b"v@?"
        )  # FIXME: Cannot test exact signature at this time
        self.assertArgIsBlock(
            Presenter.relinquishPresentedItemToWriter_, 0, b"v@?"
        )  # FIXME: Cannot test exact signature at this time

        self.assertArgIsBlock(
            Presenter.savePresentedItemChangesWithCompletionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            Presenter.accommodatePresentedItemDeletionWithCompletionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            Presenter.accommodatePresentedSubitemDeletionAtURL_completionHandler_,
            1,
            b"v@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSColorPickingHelper(AppKit.NSObject):
    def supportsMode_(self, m):
        return 0

    def provideNewView_(self, i):
        return None

    def initWithPickerMask_colorPanel_(self, m, p):
        return 1


class TestNSColorPicking(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(TestNSColorPickingHelper.supportsMode_)
        self.assertArgIsBOOL(TestNSColorPickingHelper.provideNewView_, 0)

        self.assertArgHasType(
            TestNSColorPickingHelper.initWithPickerMask_colorPanel_,
            0,
            objc._C_NSUInteger,
        )

    def testProtocols(self):
        objc.protocolNamed("NSColorPickingCustom")
        objc.protocolNamed("NSColorPickingDefault")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSNibLoading(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBundle.loadNibFile_externalNameTable_withZone_)
        self.assertResultIsBOOL(AppKit.NSBundle.loadNibNamed_owner_)
        self.assertResultIsBOOL(AppKit.NSBundle.loadNibFile_externalNameTable_withZone_)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSBundle.loadNibNamed_owner_topLevelObjects_)
        self.assertArgIsOut(AppKit.NSBundle.loadNibNamed_owner_topLevelObjects_, 2)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSClipView(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSClipView.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSClipView.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSClipView.copiesOnScroll)
        self.assertArgIsBOOL(AppKit.NSClipView.setCopiesOnScroll_, 0)
        self.assertResultIsBOOL(AppKit.NSClipView.autoscroll_)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSClipView.automaticallyAdjustsContentInsets)
        self.assertArgIsBOOL(AppKit.NSClipView.setAutomaticallyAdjustsContentInsets_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTextFieldHelper(AppKit.NSObject):
    def textField_textView_candidatesForSelectedRange_(self, tf, tv, r):
        return 1

    def textField_textView_candidates_forSelectedRange_(self, tf, tv, c, r):
        return 1

    def textField_textView_shouldSelectCandidateAtIndex_(self, tf, tv, i):
        return 1


class TestNSTextField(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextField.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSTextField.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.isBordered)
        self.assertArgIsBOOL(AppKit.NSTextField.setBordered_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.isBezeled)
        self.assertArgIsBOOL(AppKit.NSTextField.setBezeled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.isEditable)
        self.assertArgIsBOOL(AppKit.NSTextField.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.isSelectable)
        self.assertArgIsBOOL(AppKit.NSTextField.setSelectable_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.textShouldBeginEditing_)
        self.assertResultIsBOOL(AppKit.NSTextField.textShouldEndEditing_)
        self.assertResultIsBOOL(AppKit.NSTextField.acceptsFirstResponder)
        self.assertResultIsBOOL(AppKit.NSTextField.allowsEditingTextAttributes)
        self.assertArgIsBOOL(AppKit.NSTextField.setAllowsEditingTextAttributes_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.importsGraphics)
        self.assertArgIsBOOL(AppKit.NSTextField.setImportsGraphics_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSTextField.allowsDefaultTighteningForTruncation)
        self.assertArgIsBOOL(
            AppKit.NSTextField.setAllowsDefaultTighteningForTruncation_, 0
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSTextField.isAutomaticTextCompletionEnabled)
        self.assertArgIsBOOL(AppKit.NSTextField.setAutomaticTextCompletionEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSTextField.allowsCharacterPickerTouchBarItem)
        self.assertArgIsBOOL(
            AppKit.NSTextField.setAllowsCharacterPickerTouchBarItem_, 0
        )

        self.assertArgHasType(
            TestNSTextFieldHelper.textField_textView_candidatesForSelectedRange_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextFieldHelper.textField_textView_candidates_forSelectedRange_,
            3,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextFieldHelper.textField_textView_shouldSelectCandidateAtIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTextFieldHelper.textField_textView_shouldSelectCandidateAtIndex_
        )

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSTextFieldDelegate")



import CoreFoundation
from PyObjCTools.TestSupport import TestCase
import objc


class TestCFBinaryHeap(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFBinaryHeapRef)

    def testCreation(self):
        heap = CoreFoundation.CFBinaryHeapCreate(None, 0)
        self.assertIsInstance(heap, CoreFoundation.CFBinaryHeapRef)

        CoreFoundation.CFBinaryHeapAddValue(heap, "hello")
        CoreFoundation.CFBinaryHeapAddValue(heap, "world")
        CoreFoundation.CFBinaryHeapAddValue(heap, "aapjes")

        self.assertTrue(CoreFoundation.CFBinaryHeapContainsValue(heap, "hello"))
        self.assertFalse(CoreFoundation.CFBinaryHeapContainsValue(heap, "niemand"))

    def testApply(self):
        def compare(a, b, info):
            if a < b:
                return -1
            elif b < a:
                return 1
            else:
                return 0

        heap = CoreFoundation.CFBinaryHeapCreate(None, 0)
        self.assertIsInstance(heap, CoreFoundation.CFBinaryHeapRef)

        CoreFoundation.CFBinaryHeapAddValue(heap, "hello")
        CoreFoundation.CFBinaryHeapAddValue(heap, "world")
        CoreFoundation.CFBinaryHeapAddValue(heap, "aapjes")

        items = []
        contexts = []

        def function(item, context):
            items.append(item * 2)
            contexts.append(context)

        CoreFoundation.CFBinaryHeapApplyFunction(heap, function, None)
        self.assertEqual(contexts, [None, None, None])
        self.assertEqual(items, ["aapjesaapjes", "hellohello", "worldworld"])

        ctx = [""]

        def function(item, context):
            context[0] += item

        CoreFoundation.CFBinaryHeapApplyFunction(heap, function, ctx)
        self.assertEqual(ctx[0], "aapjeshelloworld")

    def testTypeID(self):
        v = CoreFoundation.CFBinaryHeapGetTypeID()
        self.assertIsInstance(v, int)

    def testCopy(self):
        heap = CoreFoundation.CFBinaryHeapCreate(None, 0)
        self.assertIsInstance(heap, CoreFoundation.CFBinaryHeapRef)

        CoreFoundation.CFBinaryHeapAddValue(heap, "hello")
        CoreFoundation.CFBinaryHeapAddValue(heap, "world")
        CoreFoundation.CFBinaryHeapAddValue(heap, "aapjes")

        heap2 = CoreFoundation.CFBinaryHeapCreateCopy(None, 0, heap)
        self.assertIsInstance(heap2, CoreFoundation.CFBinaryHeapRef)

    def testInspect(self):
        heap = CoreFoundation.CFBinaryHeapCreate(None, 0)
        self.assertIsInstance(heap, CoreFoundation.CFBinaryHeapRef)

        CoreFoundation.CFBinaryHeapAddValue(heap, "hello")
        CoreFoundation.CFBinaryHeapAddValue(heap, "world")
        CoreFoundation.CFBinaryHeapAddValue(heap, "aapjes")

        # ok, min = CoreFoundation.CFBinaryHeapGetMinimumIfPresent(heap, None)
        # self.assertTrue(ok)
        # self.assertEqual(min, "aapjes")
        # self.fail()

        count = CoreFoundation.CFBinaryHeapGetCount(heap)
        self.assertEqual(count, 3)

        count = CoreFoundation.CFBinaryHeapGetCountOfValue(heap, "hello")
        self.assertEqual(count, 1)
        count = CoreFoundation.CFBinaryHeapGetCountOfValue(heap, "fobar")
        self.assertEqual(count, 0)

        self.assertTrue(CoreFoundation.CFBinaryHeapContainsValue(heap, "hello"))
        self.assertFalse(CoreFoundation.CFBinaryHeapContainsValue(heap, "foobar"))

        min_value = CoreFoundation.CFBinaryHeapGetMinimum(heap)
        self.assertEqual(min_value, "aapjes")

        count = CoreFoundation.CFBinaryHeapGetCount(heap)
        self.assertEqual(count, 3)

        values = CoreFoundation.CFBinaryHeapGetValues(heap)
        self.assertEqual(values, ("aapjes", "hello", "world"))

        CoreFoundation.CFBinaryHeapRemoveMinimumValue(heap)
        values = CoreFoundation.CFBinaryHeapGetValues(heap)
        self.assertEqual(values, ("hello", "world"))

        CoreFoundation.CFBinaryHeapRemoveAllValues(heap)
        values = CoreFoundation.CFBinaryHeapGetValues(heap)
        self.assertEqual(values, ())

    def testFunctions(self):
        self.assertArgHasType(CoreFoundation.CFBinaryHeapGetCountOfValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBinaryHeapContainsValue, 1, b"@")
        self.assertResultHasType(CoreFoundation.CFBinaryHeapGetMinimum, b"@")
        self.assertResultHasType(
            CoreFoundation.CFBinaryHeapGetMinimumIfPresent, objc._C_NSBOOL
        )
        self.assertArgHasType(CoreFoundation.CFBinaryHeapGetMinimumIfPresent, 1, b"o^@")
        self.assertArgIsFunction(
            CoreFoundation.CFBinaryHeapApplyFunction, 1, b"v@@", False
        )
        self.assertArgHasType(CoreFoundation.CFBinaryHeapApplyFunction, 2, b"@")
        self.assertArgHasType(CoreFoundation.CFBinaryHeapAddValue, 1, b"@")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSToolbarItemGroup(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSToolbarItemGroupSelectionModeSelectOne, 0)
        self.assertEqual(AppKit.NSToolbarItemGroupSelectionModeSelectAny, 1)
        self.assertEqual(AppKit.NSToolbarItemGroupSelectionModeMomentary, 2)

        self.assertEqual(AppKit.NSToolbarItemGroupControlRepresentationAutomatic, 0)
        self.assertEqual(AppKit.NSToolbarItemGroupControlRepresentationExpanded, 1)
        self.assertEqual(AppKit.NSToolbarItemGroupControlRepresentationCollapsed, 2)

    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsSEL(
            AppKit.NSToolbarItemGroup.groupWithItemIdentifier_titles_selectionMode_labels_target_action_,  # noqa: B950
            5,
            b"v@:@",
        )
        self.assertArgIsSEL(
            AppKit.NSToolbarItemGroup.groupWithItemIdentifier_images_selectionMode_labels_target_action_,  # noqa: B950
            5,
            b"v@:@",
        )

        self.assertArgIsBOOL(AppKit.NSToolbarItemGroup.setSelected_atIndex_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbarItemGroup.isSelectedAtIndex_)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSThread(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSThread.isMultiThreaded)
        self.assertArgIsSEL(
            Foundation.NSThread.detachNewThreadSelector_toTarget_withObject_, 0, b"v@:@"
        )
        self.assertResultIsBOOL(Foundation.NSThread.setThreadPriority_)

        self.assertArgIsSEL(
            Foundation.NSThread.performSelectorOnMainThread_withObject_waitUntilDone_modes_,
            0,
            b"v@:@",
        )
        self.assertArgIsBOOL(
            Foundation.NSThread.performSelectorOnMainThread_withObject_waitUntilDone_modes_,
            2,
        )
        self.assertArgIsSEL(
            Foundation.NSThread.performSelectorOnMainThread_withObject_waitUntilDone_,
            0,
            b"v@:@",
        )
        self.assertArgIsBOOL(
            Foundation.NSThread.performSelectorOnMainThread_withObject_waitUntilDone_, 2
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsSEL(
            Foundation.NSThread.initWithTarget_selector_object_, 1, b"v@:@"
        )
        self.assertResultIsBOOL(Foundation.NSThread.isExecuting)
        self.assertResultIsBOOL(Foundation.NSThread.isFinished)
        self.assertResultIsBOOL(Foundation.NSThread.isCancelled)
        self.assertResultIsBOOL(Foundation.NSThread.isMainThread)
        self.assertResultIsBOOL(Foundation.NSThread.mainThread().isMainThread)
        self.assertArgIsSEL(
            Foundation.NSThread.performSelector_onThread_withObject_waitUntilDone_modes_,
            0,
            b"v@:@",
        )
        self.assertArgIsBOOL(
            Foundation.NSThread.performSelector_onThread_withObject_waitUntilDone_modes_,
            3,
        )
        self.assertArgIsSEL(
            Foundation.NSThread.performSelector_onThread_withObject_waitUntilDone_,
            0,
            b"v@:@",
        )
        self.assertArgIsBOOL(
            Foundation.NSThread.performSelector_onThread_withObject_waitUntilDone_, 3
        )
        self.assertArgIsSEL(
            Foundation.NSThread.performSelectorInBackground_withObject_, 0, b"v@:@"
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBlock(Foundation.NSThread.detachNewThreadWithBlock_, 0, b"v")
        self.assertArgIsBlock(Foundation.NSThread.initWithBlock_, 0, b"v")

    def testConstants(self):
        self.assertIsInstance(Foundation.NSWillBecomeMultiThreadedNotification, str)
        self.assertIsInstance(Foundation.NSDidBecomeSingleThreadedNotification, str)
        self.assertIsInstance(Foundation.NSThreadWillExitNotification, str)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSegmentedControl(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSegmentStyleAutomatic, 0)
        self.assertEqual(AppKit.NSSegmentStyleRounded, 1)
        self.assertEqual(AppKit.NSSegmentStyleTexturedRounded, 2)
        self.assertEqual(AppKit.NSSegmentStyleRoundRect, 3)
        self.assertEqual(AppKit.NSSegmentStyleTexturedSquare, 4)
        self.assertEqual(AppKit.NSSegmentStyleCapsule, 5)
        self.assertEqual(AppKit.NSSegmentStyleSmallSquare, 6)
        self.assertEqual(AppKit.NSSegmentStyleSeparated, 8)

        self.assertEqual(AppKit.NSSegmentSwitchTrackingSelectOne, 0)
        self.assertEqual(AppKit.NSSegmentSwitchTrackingSelectAny, 1)
        self.assertEqual(AppKit.NSSegmentSwitchTrackingMomentary, 2)
        self.assertEqual(AppKit.NSSegmentSwitchTrackingMomentaryAccelerator, 3)

        self.assertEqual(AppKit.NSSegmentDistributionFit, 0)
        self.assertEqual(AppKit.NSSegmentDistributionFill, 1)
        self.assertEqual(AppKit.NSSegmentDistributionFillEqually, 2)
        self.assertEqual(AppKit.NSSegmentDistributionFillProportionally, 3)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSegmentedControl.selectSegmentWithTag_)
        self.assertArgIsBOOL(AppKit.NSSegmentedControl.setSelected_forSegment_, 0)
        self.assertResultIsBOOL(AppKit.NSSegmentedControl.isSelectedForSegment_)
        self.assertArgIsBOOL(AppKit.NSSegmentedControl.setEnabled_forSegment_, 0)
        self.assertResultIsBOOL(AppKit.NSSegmentedControl.isEnabledForSegment_)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSSegmentedControl.isSpringLoaded)
        self.assertArgIsBOOL(AppKit.NSSegmentedControl.setSpringLoaded_, 0)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsSEL(
            AppKit.NSSegmentedControl.segmentedControlWithLabels_trackingMode_target_action_,
            3,
            b"v@:@",
        )
        self.assertArgIsSEL(
            AppKit.NSSegmentedControl.segmentedControlWithImages_trackingMode_target_action_,
            3,
            b"v@:@",
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBOOL(
            AppKit.NSSegmentedControl.setShowsMenuIndicator_forSegment_, 0
        )
        self.assertResultIsBOOL(AppKit.NSSegmentedControl.showsMenuIndicatorForSegment_)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSURLCredentialStorage(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSURLCredentialStorageChangedNotification, str)



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSISO8601DateFormatter(TestCase):
    @min_os_level("10.12")
    def testConstants(self):
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithYear,
            CoreFoundation.kCFISO8601DateFormatWithYear,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithMonth,
            CoreFoundation.kCFISO8601DateFormatWithMonth,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithWeekOfYear,
            CoreFoundation.kCFISO8601DateFormatWithWeekOfYear,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithDay,
            CoreFoundation.kCFISO8601DateFormatWithDay,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithTime,
            CoreFoundation.kCFISO8601DateFormatWithTime,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithTimeZone,
            CoreFoundation.kCFISO8601DateFormatWithTimeZone,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithSpaceBetweenDateAndTime,
            CoreFoundation.kCFISO8601DateFormatWithSpaceBetweenDateAndTime,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithDashSeparatorInDate,
            CoreFoundation.kCFISO8601DateFormatWithDashSeparatorInDate,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithColonSeparatorInTime,
            CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTime,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithColonSeparatorInTimeZone,
            CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTimeZone,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithFullDate,
            CoreFoundation.kCFISO8601DateFormatWithFullDate,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithFullTime,
            CoreFoundation.kCFISO8601DateFormatWithFullTime,
        )
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithInternetDateTime,
            CoreFoundation.kCFISO8601DateFormatWithInternetDateTime,
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertEqual(
            Foundation.NSISO8601DateFormatWithFractionalSeconds,
            CoreFoundation.kCFISO8601DateFormatWithFractionalSeconds,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSScrubberLayoutHelper(AppKit.NSObject):
    def scrubber_layout_sizeForItemAtIndex_(self, s, x, i):
        return 1


class TestNSScrubberLayout(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(
            AppKit.NSScrubberLayout.shouldInvalidateLayoutForSelectionChange
        )
        self.assertResultIsBOOL(
            AppKit.NSScrubberLayout.shouldInvalidateLayoutForHighlightChange
        )
        self.assertResultIsBOOL(
            AppKit.NSScrubberLayout.shouldInvalidateLayoutForChangeFromVisibleRect_toVisibleRect_  # noqa: B950
        )
        self.assertResultIsBOOL(
            AppKit.NSScrubberLayout.automaticallyMirrorsInRightToLeftLayout
        )

        self.assertResultHasType(
            TestNSScrubberLayoutHelper.scrubber_layout_sizeForItemAtIndex_,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSScrubberLayoutHelper.scrubber_layout_sizeForItemAtIndex_,
            2,
            objc._C_NSInteger,
        )

    @min_sdk_level("10.12")
    def testProtocols(self):
        objc.protocolNamed("NSScrubberFlowLayoutDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextSelection(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSTextSelectionGranularityCharacter, 0)
        self.assertEqual(AppKit.NSTextSelectionGranularityWord, 1)
        self.assertEqual(AppKit.NSTextSelectionGranularityParagraph, 2)
        self.assertEqual(AppKit.NSTextSelectionGranularityLine, 3)
        self.assertEqual(AppKit.NSTextSelectionGranularitySentence, 4)

        self.assertEqual(AppKit.NSTextSelectionAffinityUpstream, 0)
        self.assertEqual(AppKit.NSTextSelectionAffinityDownstream, 1)

    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(AppKit.NSTextSelection.isTransient)
        self.assertResultIsBOOL(AppKit.NSTextSelection.isLogical)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSATSTypesetterHelper(AppKit.NSATSTypesetter):
    def willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_(
        self, lineRect, glyphRange, usedRect, offset
    ):
        return None

    def shouldBreakLineByWordBeforeCharacterAtIndex_(self, v):
        return None

    def shouldBreakLineByHyphenatingBeforeCharacterAtIndex_(self, v):
        return True

    def hyphenationFactorForGlyphAtIndex_(self, v):
        return None

    def hyphenCharacterForGlyphAtIndex_(self, v):
        return None

    def boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_(  # noqa: B950
        self, v, v2, v3, v4, v5
    ):
        return None

    def characterRangeForGlyphRange_actualGlyphRange_(self, v1, v2):
        pass

    def glyphRangeForCharacterRange_actualCharacterRange_(self, v1, v2):
        pass

    def getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_(
        self, v1, v2, v3, v4, v5
    ):
        pass

    def setLineFragmentRect_forGlyphRange_usedRect_baselineOffset_(
        self, v1, v2, v3, v4
    ):
        pass

    def substituteGlyphsInRange_withGlyphs_(self, v1, v2):
        pass

    def insertGlyph_atGlyphIndex_characterIndex_(self, v1, v2, v3):
        pass

    def deleteGlyphsInRange_(self, v1):
        pass

    def setNotShownAttribute_forGlyphRange_(self, v1, v2):
        pass

    def setLocation_withAdvancements_forStartOfGlyphRange_(self, v1, v2, v3):
        pass

    def setAttachmentSize_forGlyphRange_(self, v1, v2):
        pass

    def setBidiLevels_forGlyphRange_(self, v1, v2):
        pass


class TestNSATSTypesetter(TestCase):
    def testByRefArguments(self):
        self.assertArgIsOut(
            AppKit.NSATSTypesetter.lineFragmentRectForProposedRect_remainingRect_, 1
        )
        self.assertArgIsInOut(AppKit.NSATSTypesetter.layoutParagraphAtPoint_, 0)
        self.assertArgIsOut(
            AppKit.NSATSTypesetter.getLineFragmentRect_usedRect_forParagraphSeparatorGlyphRange_atProposedOrigin_,  # noqa: B950
            0,
        )
        self.assertArgIsOut(
            AppKit.NSATSTypesetter.getLineFragmentRect_usedRect_forParagraphSeparatorGlyphRange_atProposedOrigin_,  # noqa: B950
            1,
        )

        o = TestNSATSTypesetterHelper.alloc().init()
        m = (
            o.willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_.__metadata__()
        )
        self.assertStartswith(m["arguments"][2]["type"], b"N^{")
        self.assertStartswith(m["arguments"][4]["type"], b"N^{")
        self.assertStartswith(m["arguments"][5]["type"], b"N^" + objc._C_CGFloat)

        m = o.shouldBreakLineByWordBeforeCharacterAtIndex_.__metadata__()
        self.assertEqual(m["retval"]["type"], objc._C_NSBOOL)
        self.assertEqual(m["arguments"][2]["type"], objc._C_NSUInteger)

        m = o.shouldBreakLineByHyphenatingBeforeCharacterAtIndex_.__metadata__()
        self.assertEqual(m["retval"]["type"], objc._C_NSBOOL)
        self.assertEqual(m["arguments"][2]["type"], objc._C_NSUInteger)

        m = o.hyphenationFactorForGlyphAtIndex_.__metadata__()
        self.assertEqual(m["retval"]["type"], objc._C_FLT)
        self.assertEqual(m["arguments"][2]["type"], objc._C_NSUInteger)

        m = o.glyphRangeForCharacterRange_actualCharacterRange_.__metadata__()
        self.assertStartswith(m["retval"]["type"], b"{")
        self.assertStartswith(m["arguments"][2]["type"], b"{")
        self.assertStartswith(m["arguments"][3]["type"], b"o^{")

        self.assertResultIsBOOL(AppKit.NSATSTypesetter.usesFontLeading)
        self.assertArgIsBOOL(AppKit.NSATSTypesetter.setUsesFontLeading_, 0)
        self.assertResultIsBOOL(AppKit.NSATSTypesetter.bidiProcessingEnabled)
        self.assertArgIsBOOL(AppKit.NSATSTypesetter.setBidiProcessingEnabled_, 0)
        self.assertArgIsBOOL(
            AppKit.NSATSTypesetter.setHardInvalidation_forGlyphRange_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSATSTypesetter.setNotShownAttribute_forGlyphRange_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSATSTypesetter.setDrawsOutsideLineFragment_forGlyphRange_, 0
        )

        self.assertArgIsIn(
            AppKit.NSATSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_, 1
        )
        self.assertArgSizeInArg(
            AppKit.NSATSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_,
            1,
            2,
        )
        self.assertArgSizeInArg(
            AppKit.NSATSTypesetter.setBidiLevels_forGlyphRange_, 0, 1
        )
        self.assertArgSizeInArg(
            AppKit.NSATSTypesetter.setBidiLevels_forGlyphRange_, 0, 1
        )

        self.assertArgIsIn(
            AppKit.NSATSTypesetter.substituteGlyphsInRange_withGlyphs_, 1
        )
        self.assertArgSizeInArg(
            AppKit.NSATSTypesetter.substituteGlyphsInRange_withGlyphs_, 1, 0
        )

    def testSubclassProtocols(self):
        self.assertResultIsBOOL(
            TestNSATSTypesetterHelper.shouldBreakLineByWordBeforeCharacterAtIndex_
        )
        self.assertResultIsBOOL(
            TestNSATSTypesetterHelper.shouldBreakLineByHyphenatingBeforeCharacterAtIndex_
        )

        self.assertResultHasType(
            TestNSATSTypesetterHelper.characterRangeForGlyphRange_actualGlyphRange_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSATSTypesetterHelper.characterRangeForGlyphRange_actualGlyphRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSATSTypesetterHelper.characterRangeForGlyphRange_actualGlyphRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )

        self.assertArgIsBOOL(
            TestNSATSTypesetterHelper.setNotShownAttribute_forGlyphRange_, 0
        )
        self.assertArgHasType(
            TestNSATSTypesetterHelper.setNotShownAttribute_forGlyphRange_,
            1,
            AppKit.NSRange.__typestr__,
        )



#
#  GraphicsBindingsDocument.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Cocoa import NSDocument, NSKeyedArchiver, NSKeyedUnarchiver, NSValueTransformer
from objc import super
from RadiansToDegreesTransformer import RadiansToDegreesTransformer


class GraphicsBindingsDocument(NSDocument):
    graphicsView = objc.IBOutlet()
    shadowInspector = objc.IBOutlet()
    graphicsController = objc.IBOutlet()
    graphics = objc.ivar()

    def init(self):
        self = super().init()
        if self is None:
            return None
        self.graphics = []  # NSMutableArray.array()
        self.bindings = []
        return self

    def windowNibName(self):
        return "GraphicsBindingsDocument"

    def makeBinding_fromObject_toObject_withKeyPath_options_(
        self, key, fromObject, toObject, withKeyPath, options
    ):
        self.bindings.append((fromObject, key))
        fromObject.bind_toObject_withKeyPath_options_(
            key, toObject, withKeyPath, options
        )

    def windowControllerDidLoadNib_(self, controller):
        super().windowControllerDidLoadNib_(controller)

        # we can't do these in IB at the moment, as
        # we don't have palette items for them

        # allow the shadow inspector (joystick) to handle multiple selections
        offsetOptions = {"NSAllowsEditingMultipleValuesSelection": True}
        angleOptions = {
            "NSValueTransformerName": "RadiansToDegreesTransformer",
            "NSAllowsEditingMultipleValuesSelection": True,
        }

        BINDINGS = [
            (
                "graphics",
                self.graphicsView,
                self.graphicsController,
                "arrangedObjects",
                None,
            ),
            (
                "selectionIndexes",
                self.graphicsView,
                self.graphicsController,
                "selectionIndexes",
                None,
            ),
            (
                "offset",
                self.shadowInspector,
                self.graphicsController,
                "selection.shadowOffset",
                offsetOptions,
            ),
            (
                "angle",
                self.shadowInspector,
                self.graphicsController,
                "selection.shadowAngle",
                angleOptions,
            ),
        ]
        for binding in BINDINGS:
            self.makeBinding_fromObject_toObject_withKeyPath_options_(*binding)

        # "fake" what should be set in IB if we had a palette...
        self.shadowInspector.maxOffset = 15

    def close(self):
        while self.bindings:
            obj, binding = self.bindings.pop()
            obj.unbind_(binding)
        super().close()

    def dataRepresentationOfType_(self, aType):
        return NSKeyedArchiver.archivedDataWithRootObject_(self.graphics)

    def loadDataRepresentation_ofType_(self, data, aType):
        self.graphics = NSKeyedUnarchiver.unarchiveObjectWithData_(data)
        return True


vt = RadiansToDegreesTransformer.alloc().init()
NSValueTransformer.setValueTransformer_forName_(vt, "RadiansToDegreesTransformer")



import Foundation
from PyObjCTools.TestSupport import TestCase

if hasattr(Foundation, "NSMachPort"):

    class TestNSMachPort(TestCase):
        def testAlloc(self):
            obj = Foundation.NSMachPort.alloc()
            self.assertIsNot(obj, None)

            obj = obj.init()
            self.assertIsNot(obj, None)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMassFormatter(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSMassFormatterUnitGram, 11)
        self.assertEqual(Foundation.NSMassFormatterUnitKilogram, 14)
        self.assertEqual(Foundation.NSMassFormatterUnitOunce, (6 << 8) + 1)
        self.assertEqual(Foundation.NSMassFormatterUnitPound, (6 << 8) + 2)
        self.assertEqual(Foundation.NSMassFormatterUnitStone, (6 << 8) + 3)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(Foundation.NSMassFormatter.isForPersonMassUse)
        self.assertArgIsBOOL(Foundation.NSMassFormatter.setForPersonMassUse_, 0)

        self.assertResultIsBOOL(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_
        )
        self.assertArgIsOut(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_, 0
        )
        self.assertArgIsOut(
            Foundation.NSLengthFormatter.getObjectValue_forString_errorDescription_, 2
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSOrderedCollectionDifference(TestCase):
    def test_constants(self):
        self.assertEqual(
            Foundation.NSOrderedCollectionDifferenceCalculationOmitInsertedObjects,
            1 << 0,
        )
        self.assertEqual(
            Foundation.NSOrderedCollectionDifferenceCalculationOmitRemovedObjects,
            1 << 1,
        )
        self.assertEqual(
            Foundation.NSOrderedCollectionDifferenceCalculationInferMoves, 1 << 2
        )

    @min_os_level("10.15")
    def test_methods(self):
        self.assertResultIsBOOL(Foundation.NSOrderedCollectionDifference.hasChanges)
        self.assertArgIsBlock(
            Foundation.NSOrderedCollectionDifference.differenceByTransformingChangesWithBlock_,
            0,
            b"@@",
        )



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDateFormatter(TestCase):
    def testOutput(self):
        formatter = Foundation.NSDateFormatter.alloc().init()
        formatter.setDateFormat_("yyyy/mm/dd")

        self.assertResultIsBOOL(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_
        )
        self.assertArgIsOut(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_, 0
        )
        self.assertArgIsInOut(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_, 3
        )
        ok, val, text_range, err = formatter.getObjectValue_forString_range_error_(
            None, "2008/10/12", Foundation.NSRange(0, 10), None
        )
        self.assertTrue(ok)
        self.assertIsInstance(val, Foundation.NSDate)
        self.assertEqual(text_range, Foundation.NSRange(0, 10))
        self.assertIs(err, None)
        self.assertResultIsBOOL(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_
        )
        self.assertArgIsInOut(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSDateFormatter.getObjectValue_forString_range_error_, 3
        )

    def testConstants(self):
        self.assertEqual(
            Foundation.NSDateFormatterNoStyle, CoreFoundation.kCFDateFormatterNoStyle
        )
        self.assertEqual(
            Foundation.NSDateFormatterShortStyle,
            CoreFoundation.kCFDateFormatterShortStyle,
        )
        self.assertEqual(
            Foundation.NSDateFormatterMediumStyle,
            CoreFoundation.kCFDateFormatterMediumStyle,
        )
        self.assertEqual(
            Foundation.NSDateFormatterLongStyle,
            CoreFoundation.kCFDateFormatterLongStyle,
        )
        self.assertEqual(
            Foundation.NSDateFormatterFullStyle,
            CoreFoundation.kCFDateFormatterFullStyle,
        )

        self.assertEqual(Foundation.NSDateFormatterBehaviorDefault, 0)
        self.assertEqual(Foundation.NSDateFormatterBehavior10_0, 1000)
        self.assertEqual(Foundation.NSDateFormatterBehavior10_4, 1040)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSDateFormatter.generatesCalendarDates)
        self.assertArgIsBOOL(Foundation.NSDateFormatter.setGeneratesCalendarDates_, 0)
        self.assertResultIsBOOL(Foundation.NSDateFormatter.isLenient)
        self.assertArgIsBOOL(Foundation.NSDateFormatter.setLenient_, 0)
        self.assertResultIsBOOL(Foundation.NSDateFormatter.isLenient)
        self.assertArgIsBOOL(
            Foundation.NSDateFormatter.initWithDateFormat_allowNaturalLanguage_, 1
        )
        self.assertResultIsBOOL(Foundation.NSDateFormatter.allowsNaturalLanguage)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(Foundation.NSDateFormatter.doesRelativeDateFormatting)
        self.assertArgIsBOOL(
            Foundation.NSDateFormatter.setDoesRelativeDateFormatting_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTableHeaderCell(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSTableHeaderCell.drawSortIndicatorWithFrame_inView_ascending_priority_,
            2,
        )

        self.assertResultHasType(
            AppKit.NSTableHeaderCell.sortIndicatorRectForBounds_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTableHeaderCell.sortIndicatorRectForBounds_,
            0,
            AppKit.NSRect.__typestr__,
        )



"""
WSTApplicationDelegateClass

An instance of this class is instantiated in the MainMenu.nib default NIB file.
"""
import objc
from Cocoa import NSObject
from WSTConnectionWindowControllerClass import WSTConnectionWindowController


class WSTApplicationDelegate(NSObject):
    @objc.IBAction
    def newConnectionAction_(self, sender):
        """Action method fired when the user selects the 'new connection'
        menu item.  Note that the WSTConnectionWindowControllerClass is
        defined the first time this method is invoked.

        This kind of lazy evaluation is generally recommended;  it speeds
        app launch time and it ensures that cycles aren't wasted loading
        functionality that will never be used.

        (In this case, it is largely moot due to the implementation of
        applicationDidFinishLaunching_().
        """
        WSTConnectionWindowController.connectionWindowController().showWindow_(sender)

    def applicationDidFinishLaunching_(self, aNotification):
        """Create and display a new connection window"""
        self.newConnectionAction_(None)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSStatusItem(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSStatusItemBehaviorRemovalAllowed, 1 << 1)
        self.assertEqual(AppKit.NSStatusItemBehaviorTerminationOnRemoval, 1 << 2)

    def testMethods(self):
        m = AppKit.NSStatusItem.setAction_.__metadata__()
        self.assertEqual(m["arguments"][2]["sel_of_type"], b"v@:@")

        m = AppKit.NSStatusItem.setDoubleAction_.__metadata__()
        self.assertEqual(m["arguments"][2]["sel_of_type"], b"v@:@")

        self.assertResultIsBOOL(AppKit.NSStatusItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSStatusItem.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSStatusItem.highlightMode)
        self.assertArgIsBOOL(AppKit.NSStatusItem.setHighlightMode_, 0)
        self.assertArgIsBOOL(
            AppKit.NSStatusItem.drawStatusBarBackgroundInRect_withHighlight_, 1
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSStatusItem.isVisible)
        self.assertArgIsBOOL(AppKit.NSStatusItem.setVisible_, 0)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class RestorationHelper(AppKit.NSObject):
    def restoreWindowWithIdentifier_state_completionHandler_(self, a, b, c):
        pass


class TestNSWindowRestoration(TestCase):
    @min_os_level("10.7")
    def testProtocol10_7(self):
        objc.protocolNamed("NSWindowRestoration")
        self.assertArgIsBlock(
            RestorationHelper.restoreWindowWithIdentifier_state_completionHandler_,
            2,
            b"v@@",
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            AppKit.NSApplication.restoreWindowWithIdentifier_state_completionHandler_
        )
        self.assertArgIsBlock(
            AppKit.NSApplication.restoreWindowWithIdentifier_state_completionHandler_,
            2,
            b"v@@",
        )

        self.assertArgIsBOOL(AppKit.NSWindow.setRestorable_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isRestorable)

        self.assertArgIsBlock(
            AppKit.NSDocument.restoreDocumentWindowWithIdentifier_state_completionHandler_,
            2,
            b"v@@",
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(
            AppKit.NSApplicationDidFinishRestoringWindowsNotification, str
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSSearchFieldCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSearchFieldRecentsTitleMenuItemTag, 1000)
        self.assertEqual(AppKit.NSSearchFieldRecentsMenuItemTag, 1001)
        self.assertEqual(AppKit.NSSearchFieldClearRecentsMenuItemTag, 1002)
        self.assertEqual(AppKit.NSSearchFieldNoRecentsMenuItemTag, 1003)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSearchFieldCell.sendsWholeSearchString)
        self.assertArgIsBOOL(AppKit.NSSearchFieldCell.setSendsWholeSearchString_, 0)
        self.assertResultIsBOOL(AppKit.NSSearchFieldCell.sendsSearchStringImmediately)
        self.assertArgIsBOOL(
            AppKit.NSSearchFieldCell.setSendsSearchStringImmediately_, 0
        )
